package pl.edu.pjwstk.yaod.yaql.parser;

import java_cup.runtime.Symbol;

%%

%public
%class Lexer
%implements Tokens
%cupsym Tokens
%unicode
%line
%column
%cup
%initthrow LexerException
%scanerror Exception
%yylexthrow LexerException
// %debug

%{
  StringBuffer string = new StringBuffer();

  public int getLine() {
    return yyline;
  }
  public int getColumn() {
    return yycolumn;
  }
  public String getText() {
    return yytext();
  }
  private Symbol token(int type) {
    return new Symbol(type, getLine(), getColumn(), getText());
  }
  
  private Symbol token(int type, Object value) {
    return new Symbol(type, getLine(), getColumn(), value);
  }

%}

%init{
//  System.out.println("Lexer ready!");
%init}

INTEGER    = [0-9]+
DOUBLE     = {INTEGER}("."{INTEGER})?
STRING     = [^\r\n\"\\]
BOOLEAN    = "true"|"false"
IDENTIFIER = [:jletter:][:jletterdigit:]*

LineTerminator      = \r|\n|\r\n
WhiteSpace          = {LineTerminator} | [ \t\f]
OneLineComment      = "//"[^\n]* {LineTerminator}
TraditionalComment  = "/*"([^*]|"*"+[^"/"])*"*"+"/"
Comment             =  {TraditionalComment} | {OneLineComment}

%state STRING

%%

<YYINITIAL> {
  ";"          { return token(SEMICOLON);      }
  "("          { return token(LROUND);         }
  ")"          { return token(RROUND);         }
  "{"          { return token(LCURLY);         }
  "}"          { return token(RCURLY);         }
  "["          { return token(LSQUARE);        }
  "]"          { return token(RSQUARE);        }

  "abs"        { return token(KEY_ABS);        }  
  "all"        { return token(KEY_ALL);        }  
  "any"        { return token(KEY_ANY);        }  
  "as"         { return token(KEY_AS);         }  
  "asc"        { return token(KEY_ASC);        }  
  "avg"        { return token(KEY_AVG);        }  
  "bag"        { return token(KEY_BAG);        }  
  "break"      { return token(KEY_BREAK);      } 
  "by"         { return token(KEY_BY);         }   
  "close"      { return token(KEY_CLOSE);      }  
  "compare"    { return token(KEY_COMPARE);    }  
  "cos"        { return token(KEY_COS);        }  
  "count"      { return token(KEY_COUNT);      }  
  "create"     { return token(KEY_CREATE);     } 
  "delete"     { return token(KEY_DELETE);     }   
  "deref"      { return token(KEY_DEREF);      }  
  "desc"       { return token(KEY_DESC);       }  
  "do"         { return token(KEY_DO);         } 
  "each"       { return token(KEY_EACH);       }   
  "else"       { return token(KEY_ELSE);       } 
  "exists"     { return token(KEY_EXISTS);     }  
  "exp"        { return token(KEY_EXP);        }  
  "for"        { return token(KEY_FOR);        }   
  "function"   { return token(KEY_FUNCTION);   }   
  "group"      { return token(KEY_GROUP);      }  
  "if"         { return token(KEY_IF);         } 
  "in"         { return token(KEY_IN);         }  
  "insert"     { return token(KEY_INSERT);     } 
  "intersect"  { return token(KEY_INTERSECT);  }  
  "into"       { return token(KEY_INTO);       } 
  "join"       { return token(KEY_JOIN);       }  
  "local"      { return token(KEY_LOCAL);      } 
  "log"        { return token(KEY_LOG);        }  
  "max"        { return token(KEY_MAX);        }  
  "min"        { return token(KEY_MIN);        }  
  "minus"      { return token(KEY_MINUS);      }  
  "order"      { return token(KEY_ORDER);      }  
  "permanent"  { return token(KEY_PERMANENT);  } 
  "print"      { return token(KEY_PRINT);      }
  "procedure"  { return token(KEY_PROCEDURE);  } 
  "rename"     { return token(KEY_RENAME);     } 
  "return"     { return token(KEY_RETURN);     } 
  "sequence"   { return token(KEY_SEQUENCE);   }  
  "show"       { return token(KEY_SHOW);       }
  "sign"       { return token(KEY_SIGN);       }  
  "sin"        { return token(KEY_SIN);        }  
  "sqrt"       { return token(KEY_SQRT);       }  
  "struct"     { return token(KEY_STRUCT);     }  
  "sum"        { return token(KEY_SUM);        }  
  "symmetric"  { return token(KEY_SYMMETRIC);  }  
  "to"         { return token(KEY_TO);         }   
  "toBag"      { return token(KEY_TOBAG);      }  
  "toDouble"   { return token(KEY_TODOUBLE);   }  
  "toInteger"  { return token(KEY_TOINTEGER);  }  
  "toSequence" { return token(KEY_TOSEQUENCE); }  
  "toSingle"   { return token(KEY_TOSINGLE);   }  
  "toString"   { return token(KEY_TOSTRING);   }  
  "toStruct"   { return token(KEY_TOSTRUCT);   }  
  "union"      { return token(KEY_UNION);      }  
  "unique"     { return token(KEY_UNIQUE);     }  
  "update"     { return token(KEY_UPDATE);     } 
  "where"      { return token(KEY_WHERE);      }  
  "while"      { return token(KEY_WHILE);      }   
  "xml"        { return token(KEY_XML);        }

//  "like"       { return token(KEY_LIKE);       }
//  "relike"     { return token(KEY_RELIKE);     }
//  "toBOOLEAN"  { return token(KEY_TOBOOLEAN);  }


  "="          { return token(OP_ASSIGN);        }
  "+="         { return token(OP_PLUS_ASSIGN);   }
  "-="         { return token(OP_MINUS_ASSIGN);  }
  "*="         { return token(OP_TIMES_ASSIGN);  }
  "/="         { return token(OP_DIVIDE_ASSIGN); }
  "%="         { return token(OP_MODULO_ASSIGN); }
  "++"         { return token(OP_INCREMENT);     }
  "--"         { return token(OP_DECREMENT);     }
  ".."         { return token(OP_RANGE);         }
  "?"          { return token(OP_QUESTION);      }
  ":"          { return token(OP_COLON);         }
  "&&"         { return token(OP_CONJUNCTION);   }
  "||"         { return token(OP_DISJUNCTION);   }
  "!"          { return token(OP_NEGATION);      }
  "=="         { return token(OP_EQUAL);         }
  "!="         { return token(OP_NOT_EQUAL);     }
  "<"          { return token(OP_LOWER);         }
  "<="         { return token(OP_LOWER_EQUAL);   }
  ">"          { return token(OP_GREATER);       }
  ">="         { return token(OP_GREATER_EQUAL); }
  "+"          { return token(OP_PLUS);          }
  "-"          { return token(OP_MINUS);         }
  "*"          { return token(OP_TIMES);         }
  "/"          { return token(OP_DIVIDE);        }
  "%"          { return token(OP_MODULO);        }
  "^"          { return token(OP_POWER);         }
  ","          { return token(OP_COMMA);         }
  "."          { return token(OP_NAVIGATION);    }
  "<-"         { return token(OP_INSERT);        }

  {BOOLEAN}    { return token(LIT_BOOLEAN, yytext()); }
  {INTEGER}    { return token(LIT_INTEGER, yytext()); }
  {DOUBLE}     { return token(LIT_DOUBLE,  yytext()); }
  \"           { string.setLength(0); yybegin(STRING); }
  
  {WhiteSpace} { /* ignore white spaces */ }  
  {Comment}    { /* ignore comments     */ }
}

<YYINITIAL> {
  {IDENTIFIER} { return token(IDENTIFIER,  yytext()); }
}

<STRING> {
  \"                  { yybegin(YYINITIAL); return token(LIT_STRING, string.toString()); }
  {STRING}            { string.append(yytext());  }
  "\\b"               { string.append('\b');      }
  "\\t"               { string.append('\t');      }
  "\\n"               { string.append('\n');      }
  "\\f"               { string.append('\f');      }
  "\\r"               { string.append('\r');      }
  "\\\""              { string.append('\"');      }
  "\\\'"              { string.append('\'');      }
  "\\\\"              { string.append('\\');      }
  \\.                 { throw new LexerException("LEXER: Illegal escape sequence \""+yytext()+"\"", yyline, yycolumn); }
  {LineTerminator}    { throw new LexerException("LEXER: Unterminated string at end of line", yyline, yycolumn); }
}
<YYINITIAL> {
.|\n   { throw new LexerException("LEXER: Illegal character <" + yytext() + ">", yyline, yycolumn); }
//[^] { throw new LexerException("LEXER: Illegal character <" + yytext() + ">", yyline, yycolumn); }

//  "quit" {System.out.println("\n!!! Quit !!!"); yyclose(); /* System.exit(0); */ }
}