package pl.edu.pjwstk.yaod.atoms;

import java.util.*;
import java.io.*;

import pl.edu.pjwstk.yaod.util.*;
import pl.edu.pjwstk.yaod.atoms.*;

/**
 * Klasa serializujaca atomy. Tymczasowo wykorzystywany do tego
 * celu jest mechanizm serializacji Javy. W przyszlosci trzeba
 * w tablicy byte[] ustawic w odpowiedniej poszczegolne wartosci atomu.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */
public class AtomSerializer {
  private ObjectSerializer m_objectSerializer = new ObjectSerializer();

  public AtomSerializer() {
  }

/**
 * Deserializuje atom zserializowany za pomoca mechanizmu serializacji Javy.
 * @param rawAtm zserializowany atom.
 * @return zdeserializowany atom.
 */
  public Atom atomDeserialize(byte[] rawAtm) {
    return (Atom) m_objectSerializer.objectDeserialize(rawAtm);    
  }

/**
 * Serializuje atom za pomoca mechanizmu serializacji Javy.
 * @param atm atom do zserializowania.
 * @return tablica zawierajaca zserializowany atom.
 */
  public byte[] atomSerialize(Atom atm) {
    return m_objectSerializer.objectSerialize(atm);
  }

/**
 * Serializuje atom za pomoca wlasnego mechanizmu serializacji.
 * @param atm atom do zserializowania.
 * @return zserializowany atom.
 * TODO: metoda nie jest skonczona.
 */
  public byte[] atomSerialize2(Atom atm) {
    RawAtom rawAtm = new RawAtom();

    // serializacja typu i wartosci
    if (atm instanceof StringAtom) {
      rawAtm.atmType = ATOM_STRING;
      rawAtm.atmValue = ((StringAtom) atm).getValue().getBytes();
    }
    else if (atm instanceof SpiderAtom) {
      rawAtm.atmType = ATOM_SPIDER;
      for (Enumeration e = ((SpiderAtom) atm).getMembers(); e.hasMoreElements(); ) {
//        (AtomID) 
      }
    }
    else if (atm instanceof BackwardAtom) {
      rawAtm.atmType = ATOM_BACKWARD;
      for (Enumeration e = ((BackwardAtom) atm).getBackwardRefs(); e.hasMoreElements(); ) {
// 
      }
    }

    // serializacja nazwy
    rawAtm.atmName = atm.getName().getBytes();

    // serializacja linkow
    if (atm.getOwner() != null) {
      rawAtm.atmLinks |= 1;
      
    }
    if (atm.getNext() != null) {
      rawAtm.atmLinks |= 2;
      
    }
    if (atm.getPrevious() != null) {
      rawAtm.atmLinks |= 4;
      
    }

    return rawAtm.toByteArray();
  }

/**
 * Deserializuje atom za pomoca wlasnego mechanizmu serializacji.
 * @param atmAttr zserializowany atom.
 * @return zdeserializowany atom.
 * TODO: metoda nie jest zaimplementowana.
 */
  public Atom atomDeserialize2(byte[] atmArr) {
    return null;
  }

/**
 * Serializuje identyfikator atomu.
 * @param aid identyfikator atomu do zserializowania.
 * @return zserializowany atom.
 * TODO: metoda nie jest zaimplementowana.
 */
  public byte[] serializeAtomID(AtomID aid) {
    return null;
  }

/**
 * Deserializuje identyfikator atomu.
 * @param idAttr zserializowany atom.
 * @return zdeserializowany atom.
 * TODO: metoda nie jest zaimplementowana.
 */
  public AtomID deserializeAtomID(byte[] idArr) {
    return null;
  }

/**
 * Klasa pomocnicza, reprezentujaca atom zserializowany
 * za pomoca wlasnego mechanizmu serializacji.
 * TODO: klasa nie jest skonczona.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */
  class RawAtom {
    byte atmType = 0;
    byte atmLinks = 0;
    byte[] atmName;
    byte[] atmPtrs = null;
    byte[] atmValue = null;

    byte[] toByteArray() {
      return null;
    }
  }

  private final static byte ATOM_BACKWARD = 1;
  private final static byte ATOM_DOUBLE = 2;
  private final static byte ATOM_INTEGER = 3;
  private final static byte ATOM_REFERENCE = 4;
  private final static byte ATOM_SNAKE = 5;
  private final static byte ATOM_SPIDER = 6;
  private final static byte ATOM_STRING = 7;
  private final static byte ATOM_TURTLE = 8;
}
