package pl.edu.pjwstk.yaod.atoms;

import java.util.*;

/**
 * Atom referencji zwrotnych. Jego zadaniem jest przechowywanie
 * identyfikatorow wszystkich referencji do danego obiektu.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class BackwardAtom extends Atom {
  private Vector m_backwardRefs;

/**
 * Tworzy nowy atom referencji zwrotnych.
 * @return nowy atom referencji zwrotnych.
 */
  public BackwardAtom() {
    super("$BACKWARD");
    m_backwardRefs = new Vector();
  }

/**
 * Rejestruje identyfikator referencji, tworzac referencje zwrotna.
 * @param ptr identyfikator referencji.
 */
  public void addBackwardRef(AtomID ptr) {
    m_backwardRefs.addElement(ptr);
    setModified(true);
  }

/**
 * Usuwa referencje zwrotna, reprezentujaca referencje o podanym identyfikatorze.
 * @param ptr identyfikator referencji.
 */
  public void deleteBackwardRef(AtomID ptr) {
    m_backwardRefs.remove(ptr);
    setModified(true);
  }

/**
 * Zwraca wszystkie referencje zwrotne zarejestrowane w danym atomie.
 * @return obiekt pozwalajacy na dostep do referencji zwrotnych.
 */
  public Enumeration getBackwardRefs() {
    return m_backwardRefs.elements();
  }

/**
 * Zwraca pierwsza referencje zwrotna.
 * @return pierwsza referencja zwrotna.
 */
  public AtomID getFirstBackwardRef() {
    return m_backwardRefs.isEmpty() ? null : (AtomID) m_backwardRefs.firstElement();
  }

/**
 * Zlicza wszystkie referencje zwrotne zarejestrowane w atomie.
 * @return ilosc referencji.
 */
  public int countBackwardRefs() {
    return m_backwardRefs.size();
  }

}
