package pl.edu.pjwstk.yaod.atoms;

import java.lang.reflect.*;

import pl.edu.pjwstk.yaod.atoms.*;

/**
 * Klasa reprezentujaca atom skladu obiektow javy. Pozwala na podpiecie
 * do tego skladu dowolnego obiektu Double Javy.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class JavaDoubleAtom extends DoubleAtom {
  private Object m_JavaObject;
  private String m_JavaAttribute;
  private Field m_JavaField;

/**
 * Tworzy nowy atom.
 * @param name nazwa atomu.
 * @param obj obiekt zawierajacy atrybut majacy byc wartoscia atomu.
 * @param attr nazwa atrybutu obiektu obj, ktorego wartosc ma byc wartoscia atomu.
 * @exception NoSuchFieldException nazwa atrybutu nie istnieje w podanym obiekcie.
 * @return nowy atom.
 */
  public JavaDoubleAtom(String name, Object obj, String attr) throws NoSuchFieldException {
    super(name, 0);

    m_JavaObject = obj;
    m_JavaAttribute = attr;
    m_JavaField = m_JavaObject.getClass().getField(attr);

    if (m_JavaField.getType() != double.class) 
      throw new NoSuchFieldException();
  }

/**
 * Ustala wartosc atomu (obiektu Javy).
 * @param val nowa wartosc atomu.
 */
  public void setValue(double val) {
    try {
      m_JavaField.setDouble(m_JavaObject, val);
    }
    catch (IllegalAccessException e) {
      System.out.println("access error");
    }
  }

/**
 * Zwraca wartosc atomu.
 * @return wartosc atomu.
 */
  public double getValue() {
    double res = 0;
    try {
      res = m_JavaField.getDouble(m_JavaObject);
    }
    catch (IllegalAccessException e) {
      System.out.println("access error");
    }
    return res;
  }  
}
