package pl.edu.pjwstk.yaod.atoms;

import java.util.*;

/**
 * Klasa reprezentujaca atom zlozony typu spider. Atom zawiera
 * referencje do wszystkich obiektow potomnych, a wszystkie obiekty
 * potomne zawieraja referencje do atomu nadrzednego.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class SpiderAtom extends Atom {
  private Vector m_members;

/**
 * Tworzy nowy obiekt typu spider.
 * @param name nazwa atomu.
 * @return nowy atom.
 */
  public SpiderAtom(String name) {
    super(name);
    m_members = new Vector();
  }

/**
 * Odrejestrowywuje atom porzedny.
 * @param delPtr identyfikator odrejestrowywanego atomu.
 */
  public void removeMember(AtomID delPtr) {
    for (Enumeration e = m_members.elements(); e.hasMoreElements(); ) {
      AtomID ptr = (AtomID)e.nextElement();
      if (ptr.equals(delPtr)) {
        m_members.removeElement(ptr);
        break;
      }
    }
    setModified(true);
  }

/**
 * Rejestruje atom podrzedny jako poprzedzajacy podany atom.
 * @param member identyfikator rejestrowanego atomu.
 * @param before identyfikator zarejestrowanego atomu, ktory ma byc nastepny.
 */
  public void connectMemberBefore(AtomID member, AtomID before) {
    int idx = 0;
    for (Enumeration e = m_members.elements(); e.hasMoreElements(); idx++) {
      AtomID ptr = (AtomID)e.nextElement();
      if (ptr.equals(before))
        m_members.insertElementAt(member, idx);   
    }
    setModified(true);
  }

/**
 * Rejestruje atom podrzedny jako nastepny w stosunku do podanego atomu.
 * @param member identyfikator rejestrowanego atomu.
 * @param after identyfikator zarejestrowanego atomu, ktory ma byc poprzedni.
 */
  public void connectMemberAfter(AtomID member, AtomID after) {
    int idx = 0;
    for (Enumeration e = m_members.elements(); e.hasMoreElements(); idx++) {
      AtomID ptr = (AtomID)e.nextElement();
      if (ptr.equals(after))
        m_members.insertElementAt(member, idx + 1);
    }
    setModified(true);
  }

/**
 * Zwraca identyfikator atomu nastepnego w stosunku do podanego atomu.
 * @param prevPtr atom poprzedni.
 * @return identyfikator nastepnego atomu.
 */
  public AtomID getNextMember(AtomID prevPtr) {
    Enumeration e = m_members.elements();
    while (e.hasMoreElements()) {
      AtomID ptr = (AtomID)e.nextElement();
      if (ptr.equals(prevPtr))
        break;
    }

    return e.hasMoreElements() ? (AtomID)e.nextElement() : null;
  }

/**
 * Zwraca identyfikator pierwszego atomu porzednego.
 * @return identyfikator pierwszego atomu.
 */
  public AtomID getFirstMember() {
    return m_members.isEmpty() ? null : (AtomID)m_members.firstElement();
  }

/**
 * Zwraca identyfikatory wszystkich atomow podrzednych.
 * @return obiekt umozliwiajacy dostep do identyfikatorow.
 */
  public Enumeration getMembers() {
    return m_members.elements();
  }

/**
 * Zlicza wszystkie zarejestrowane atomy podrzedne.
 * @return liczba atomow podrzednych.
 */
  public int countMembers() {
    return m_members.size();
  }

/**
 * Rejestruje atom porzedny jako ostatni.
 * @param ptr identyfikator rejestrowanego atomu.
 */
  public void addMember(AtomID ptr) {
    m_members.addElement(ptr);
    setModified(true);
  }

/**
 * Zwraca identyfikator atomu podrzednego na podanej pozycji.
 * @param i indeks atomu porzednego.
 * @return identyfikator atomu porzednego.
 */
  public AtomID getMemberAt(int i) {
    return (AtomID)m_members.elementAt(i);
  }
}
