package pl.edu.pjwstk.yaod.debug;

import java.util.*;

import javax.swing.tree.*;

import pl.edu.pjwstk.yaod.objects.*;
import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.system.processes.*;

/**
 * Klasa tworzy obiekt DefaultMutableTreeNode odwzorowujacy
 * zawartosc bazy danych (hierarchiczna struktura atomow).
 * Obiekt ten moze byc nastepnie przeslany do klienta
 * i wyswietlony w JTree.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class AtomExtractor {
  public DefaultMutableTreeNode dumpStore(SVRP svrp, short id) throws NoSuchObjectException {
    Wrappable store = svrp.getStoreByID(id);
    DefaultMutableTreeNode root = new DefaultMutableTreeNode("Store's dump");

    dumpComplex(svrp, store.getEntry(), root);

    return root;
  }

  private DefaultMutableTreeNode dumpComplex(SVRP svrp, ObjectID oid, DefaultMutableTreeNode root)
      throws NoSuchObjectException {
    YComplexObject cpx = (YComplexObject) svrp.getObjectManager().getObject(oid);

    StringBuffer cpxValue = new StringBuffer();
    for (Enumeration e = cpx.getMembers(); e.hasMoreElements(); ) {
      cpxValue.append( ((ObjectID) e.nextElement()).toString() );
      if (e.hasMoreElements())
        cpxValue.append(", ");        
    }

    DefaultMutableTreeNode cpxNode =
      new DefaultMutableTreeNode("<" + cpx.getOID().toString() +
                                 ", " + cpx.getName() + ", {" + cpxValue.toString() + "}>");
    root.add(cpxNode);
    
    for (Enumeration e = cpx.getMembers(); e.hasMoreElements(); ) {
      ObjectID mid = (ObjectID) e.nextElement();
      YObject obj = (YObject) svrp.getObjectManager().getObject(mid);
      if (obj instanceof YComplexObject)
        dumpComplex(svrp, mid, cpxNode);
      else {
        String nodeStr = "<" + obj.getOID().toString() +
                                     ", " + obj.getName() + ", "; 
        if (obj instanceof YSimpleObject) {
          if (obj instanceof YString)
            nodeStr += "\"" + ((YString) obj).getValue() + "\">";
          else
            nodeStr += ((YSimpleObject) obj).stringValue() + ">";
        }
        else if (obj instanceof YReference)
          nodeStr += ((YReference) obj).getValue().toString() + ">";

        DefaultMutableTreeNode mbrNode = new DefaultMutableTreeNode(nodeStr);

        cpxNode.add(mbrNode);
      }
    }

    return cpxNode;
  }
}
