package pl.edu.pjwstk.yaod.network.protocol;

import java.util.*;

import javax.swing.tree.*;

/**
 * Klasa reprezentuje odpowiedz serwera na wykonane zapytanie.
 * Wynik zapytania jest drzewem zawierajacym obiekty odpowiadajace
 * rezultatom zapytan w sba (np. bag, sequence itp).
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class QueryReply extends ServerReply {
  private Vector m_results;
  private DefaultMutableTreeNode m_asts;
  private DefaultMutableTreeNode m_envs;

/**
 * Tworzy nowy obiekt odpowiedzi, ktory zostanie przeslany do klienta.
 * @param results wyniki zapytania.
 * @param asts drzewa ast poszczegolnych instrukcji.
 * @param envs zawartosc stosu srodowiskowego po wykonaniu zapytania.
 * @return nowy obiekt odpowiedzi.
 */
  public QueryReply(Vector results, DefaultMutableTreeNode asts, DefaultMutableTreeNode envs) {
    super("");
    m_results = results;
    m_asts = asts;
    m_envs = envs;
  }

/**
 * Zwraca wyniki instrukcji bedacych czescia zapytania.
 * @return wyniki zapytania.
 */
  public Enumeration getResults() {
    return m_results.elements();
  }

/**
 * Zwraca obiekt reprezentujacy drzewo skladniowe przeslanego zapytania.
 * @return obiekt reprezentujacy ast.
 */
  public DefaultMutableTreeNode getAst() {
    return m_asts;
  }

/**
 * Zwraca obiekt reprezentujacy zawartosc stosu srodowiskowego
 * po wykonaniu zapytania.
 * @return obiekt reprezentujacy zawartosc envs.
 */
  public DefaultMutableTreeNode getEnvs() {
    return m_envs;
  }
}
