package pl.edu.pjwstk.yaod.objects;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.system.processes.*;

/**
 * Klasa, ktorej zadaniem jest dodanie zachowania do statycznych struktur,
 * jakimi sa atomy referencyjne. Atomy te maja za zadanie przechowywanie
 * referencji zwrotnych, ktore wykorzystywane sa do utrzymania integralnosci
 * bazy danych po skasowaniu obiektu (odpowiednik garbage collectora w javie).
 * 
 * @author M. Lentner
 * @version 1.0
 */

public class YBackward extends YObject {
/**
 * Konstruktor tworzacy nowy obiekt typu YBackward. 
 * @param sp proces uslugowy, zapewniajacy uslugi dla tego obiektu.
 * @return nowy obiekt referencji zwrotnych.
 */
  YBackward(SVRP sp) {
    super(sp, new BackwardAtom());
  }

/**
 * Konstruktor tworzacy nowy obiekt typu YBackward.
 * @param sp proces uslugowy, zapewniajacy uslugi dla tego obiektu.
 * @return nowy obiekt referencji zwrorntch
 */

  YBackward(SVRP sp, BackwardAtom atom) {
    super(sp, atom);
  }

/**
 * Rejestruje referencje zwrotna.
 * @param refID identyfikator obiektu referencyjnego, ktorego utworzenie
 * spowodowalo koniecznosc zarejestrowania go w atomie referencji zwrotnych.
 */

  void addBackwardRef(ObjectID refID) {
    ((BackwardAtom) getAtom()).addBackwardRef((AtomID) refID);
  }

/**
 * Usuwa referencje zwrotna, odrejestrowywujac ja ze stosownego atomu.
 * @param delRef identyfikator usuwanej referencji.
 */

  void deleteBackwardRef(ObjectID delRef) {
    BackwardAtom backwardAtm = (BackwardAtom) getAtom();
    backwardAtm.deleteBackwardRef((AtomID) delRef);

    if (backwardAtm.countBackwardRefs() == 0) {
      try {
        super.disconnect();
      }
      catch (NoSuchObjectException e) {
        System.out.println("deleteBackwardRef()");
        e.printStackTrace();
      }
    }
  }

/**
 * Usuwa obiekt referencji zwrotnych. Przed usunieciem samego obiektu,
 * kasuje rowniez zarejestrowane w nim obiekty referencji zwrotnych.
 */

  public void disconnect() throws NoSuchObjectException {
    BackwardAtom backwardAtm = (BackwardAtom) getAtom();

    ObjectID refID;

    while ( (refID = backwardAtm.getFirstBackwardRef()) != null) {
      YObject obj = getServer().getObjectManager().getObject(refID);
      obj.disconnect();
    }
  }
}
