package pl.edu.pjwstk.yaod.objects;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.system.processes.*;

/**
 * Klasa reprezentujaca obiekt zawierajacy dane binarne.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class YBinary extends YSimpleObject {
/**
 * Tworzy nowy obiekt danych binarnych.
 * @param sp proces uslugowy, ktory bedzie wykonywal operacje dla tego obiektu.
 * @param name nazwa obiektu.
 * @param value wartosc obiektu.
 * @return nowy obiekt danych binarnych.
 */
  public YBinary(SVRP sp, String name, byte[] value) {
    super(sp, new BinaryAtom(name, value));
  }

/**
 * Tworzy nowy obiekt danych binarnych na bazie atomu juz zapisanego
 * w bazie.
 * @param sp proces uslugowy, ktory bedzie wykonywal operacje dla tego obiektu.
 * @return nowy obiekt danych binarnych.
 */
  public YBinary(SVRP sp, BinaryAtom atom) {
    super(sp, atom);
  }

/**
 * Zwraca wartosc obiektu.
 * @return wartosc obiektu.
 */
  public byte[] getValue() {
    return ((BinaryAtom) getAtom()).getValue();
  }

/**
 * Ustala wartosc obiektu.
 * @param val nowa wartosc obiektu.
 */
  public void setValue(byte[] val) {
    ((BinaryAtom) getAtom()).setValue(val);
  }

/**
 * Konwertuje wartosc obiektu do ciagu znakow.
 * @return wartosc obiektu jako ciag znakow.
 */
  public String stringValue() {
    return new String(getValue());
  }
}
