package pl.edu.pjwstk.yaod.objects;

import java.util.*;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.store.defaultstore.blocks.*;
import pl.edu.pjwstk.yaod.system.processes.*;

/**
 * Klasa z ktorej dziedzicza wszystkie obiekty zlozone.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public abstract class YComplexObject extends YObject {
/**
 * Tworzy nowy obiekt zlozony.
 * @param sp proces uslugowy, ktory ma za zadanie wykonywac operacje
 * dla danego obiektu.
 * @return nowy obiekt zlozony.
 */
  YComplexObject(SVRP sp, Atom atm) {
    super(sp, atm);
  }

/**
 * Podpina nowy obiekt podrzedny, utrwalajac go w skladzie
 * oraz zapisujac jego identyfikator.
 * @param member podpinany obiekt.
 * @exception BlockFreeSpaceException brak wolnego miejsca.
 */
  abstract void connectMember(YObject memberAtm) throws BlockFreeSpaceException;

/**
 * Podpina nowy obiekt podrzedny jako nastepnik istniejacego obiektu.
 * @param member obiekt majacy zostac podpiety do skladu.
 * @param base identyfikator obiektu wzgledem ktorego jako nastepny
 * ma zostac podpiety dany obiekt.
 * @exception BlockFreeSpaceException brak wolnego miejsca.
 * @return comments
 */
  abstract void connectMemberAfter(YObject member, ObjectID base) throws BlockFreeSpaceException;

/**
 * Podpina nowy obiekt podrzedny jako poprzednik istniejacego obiektu.
 * Podpiecie oznacza utworzenie obiektu w skladzie i zarejestrowanie go
 * jako obiekt podrzedny.
 * @param member podpinany obiekt.
 * @param base identyfikator obiektu wzgledem ktorego jako poprzedni
 * ma zostac podpiety dany obiekt.
 * @exception BlockFreeSpaceException brak wolnego miejsca.
 * @param base identyfikator obiektu przed ktorym umiescic nowy obiekt podrzedny.
 */
  abstract void connectMemberBefore(YObject member, ObjectID base) throws BlockFreeSpaceException;

/**
 * Rozlacza obiekt podrzedny, odrejestrowywujac jego identyfikator
 * oraz kasujac atom ze skladu.
 * @param member identyfikator usuwanego obiektu.
 * @exception NoSuchObjectException proba usuniecia nieistniejacego
 * obiektu podrzednego.
 */
  abstract void disconnectMember(ObjectID member) throws NoSuchObjectException;

/**
 * Zwraca wszystkie obiekty podrzedne.
 * @return obiekt zawierajacy identyfikatory wszystkich obiektow
 * podrzednych.
 */
  public abstract Enumeration getMembers();

/**
 * Zwraca identyfikatory wszystkich obiektow lezacych na tym
 * samym poziomie w hierarchii obiektow.
 * @return obiekt zawierajacy identyfikatory.
 */
  public abstract Enumeration getSubordinates();

/**
 * Zwraca nastepny obiekt podrzedny w stosunku do podanego.
 * @param startID identyfikator obiektu bazowego.
 * @return oid nastepnego obiektu.
 */
  public abstract ObjectID getNextMember(ObjectID member);

/**
 * Zwraca identyfikator poprzedniego obiektu w grafie obiektow.
 * TODO: do zaimplementowania
 * @param member oid bazowego obiektu.
 * @return oid poprzedniego obiektu.
 */
  public abstract ObjectID getPreviousMember(ObjectID member);

/**
 * Rejestruje obiekt podrzedny.
 * @param ptr identyfikator rejestrowanego obiektu.
 */
  abstract void registerMember(ObjectID ptr);

/**
 * Odrejestrowywuje obiekt podrzedny.
 * @param ptr identyfikator odrejestrowywanego obiektu.
 */
  abstract void unregisterMember(ObjectID ptr);

/**
 * Zwraca pierwszy obiekt podrzedny o podanym typie.
 * TODO: do zaimplementowania.
 * @param type typ obiektu.
 * @param startPtr identyfikator obiektu, od ktorego zaczac poszukiwania.
 * @return oid znalezionego obiektu.
 */
  public abstract ObjectID findMemberByNameAndType(String name, byte type, ObjectID currPtr);

}
