package pl.edu.pjwstk.yaod.objects;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.store.defaultstore.blocks.*;
import pl.edu.pjwstk.yaod.system.processes.*;
import pl.edu.pjwstk.yaod.util.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.*;

/**
 * Obiekt reprezentujacy procedure w skladzie obiektow.
 * Procedura jest obiektem typu spider z trzema obiektami potomnymi
 * zawierajacymi zrodlo, ast zawartosci i ast parametrow.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class YProcedure extends YSpider {
/**
 * Tworzy nowy obiekt procedury.
 * @param sp proces uslugowy wykonujacy operacje dla obiektu.
 * @param name nazwa procedury.
 * @return nowy obiekt procedury.
 */
  public YProcedure(SVRP sp, String name) {
    super(sp, new ProcedureAtom(name));
  }

/**
 * Tworzy nowy obiekt procedury na bazie istniejacego juz w skladzie atomu.
 * @param sp proces uslugowy wykonujacy operacje dla obiektu.
 * @exception atm atom procedury.
 * @return nowy obiekt procedury.
 */
  public YProcedure(SVRP sp, ProcedureAtom atm) {
    super(sp, atm);
  }

/**
 * Zwraca ast ciala procedury.
 * @exception NoSuchObjectException brak obiektu podrzednego o nazwie ast.
 * @return ast zawierajace cialo procedury.
 */
  public AbstractSyntaxTree getBody() throws NoSuchObjectException {
    ObjectID astid = this.findMemberByName("BODY", null);
    if (astid == null) return null;

    YBinary astobj = (YBinary) getServer().getObjectManager().getObject(astid);

    byte[] raw = astobj.getValue();

    AbstractSyntaxTree ast =
      (AbstractSyntaxTree) new ObjectSerializer().objectDeserialize(raw);

    return ast;
  }

/**
 * Ustala ast ciala procedury.
 * @param ast 
 * @exception XxxxxxException if ...
 * @return comments
 */
  public void setBody(AbstractSyntaxTree ast)
      throws NoSuchObjectException, BlockFreeSpaceException {
    ObjectID astid = this.findMemberByName("BODY", null);

    if (astid == null) {
      YBinary m_ast = getServer().getObjectManager().createBinary("BODY", null);
      m_ast.connect(getOID());

      astid = m_ast.getOID();
    }
    
    YBinary astobj = (YBinary) getServer().getObjectManager().getObject(astid);

    if (ast == null) {
      astobj.disconnect();
      return;
    }

    byte[] raw = new ObjectSerializer().objectSerialize(ast);

    astobj.setValue(raw);
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public void setParameters(AbstractSyntaxTree ast)
      throws NoSuchObjectException, BlockFreeSpaceException {
    ObjectID astid = findMemberByName("PARAMETERS", null);

    if (astid == null) {
      YBinary m_par = getServer().getObjectManager().createBinary("PARAMETERS", null);
      m_par.connect(getOID());

      astid = m_par.getOID();
    }
    
    YBinary astobj = (YBinary) getServer().getObjectManager().getObject(astid);

    if (ast == null) {
      astobj.disconnect();
      return;
    }

    byte[] raw = new ObjectSerializer().objectSerialize(ast);

    astobj.setValue(raw);
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public AbstractSyntaxTree getParameters() throws NoSuchObjectException {
    ObjectID astid = this.findMemberByName("PARAMETERS", null);

    if (astid == null) return null;

    YBinary astobj = (YBinary) getServer().getObjectManager().getObject(astid);

    byte[] raw = astobj.getValue();

    AbstractSyntaxTree ast =
      (AbstractSyntaxTree) new ObjectSerializer().objectDeserialize(raw);

    return ast;
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public String getSource() throws NoSuchObjectException {
    ObjectID srcid = findMemberByName("SOURCE", null);
    
    if (srcid == null) return null;
    
    YString src = (YString) getServer().getObjectManager().getObject(srcid);

    return src.getValue();
  }

/**
 * 
 * @param paramName source 
 * @exception NoSuchObjectException 
 */
  public void setSource(String source) throws NoSuchObjectException, BlockFreeSpaceException {
    ObjectID srcid = findMemberByName("SOURCE", null);

    if (srcid == null) {
      YString m_src = getServer().getObjectManager().createString("SOURCE", "");
      m_src.connect(getOID());

      srcid = m_src.getOID();
    }

    YString src = (YString) getServer().getObjectManager().getObject(srcid);

    if (source == null) {
      src.disconnect();
      return;
    }

    src.setValue(source);
  }
}
