package pl.edu.pjwstk.yaod.store;

import pl.edu.pjwstk.yaod.system.*;
import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.system.buffers.*;

/**
 * Klasa z ktorej dziedzicza wszystkie implementacje blokow
 * (glownie ze wzgledu na bufor blokow i dbwr).
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public abstract class AbstractBlock implements Bufferable {
  protected AbstractBlockID m_blockID;
  private boolean m_modified;

/**
 * Tworzy nowy blok.
 */
  public AbstractBlock() {
  }

/**
 * Zwraca identyfikator bloku.
 * @return identyfikator bloku.
 */
  public AbstractBlockID getBlockID() {
    return m_blockID;
  }

/**
 * Ustala identyfikator bloku.
 * @param blockID nowy identyfikator bloku.
 */
  public void setBlockID(AbstractBlockID blockID) {
    m_blockID = blockID;
  }

/**
 * Ustala flage modyfikacji, ktora pomaga w decyzji czy blok wymaga
 * zapisania na dysku.
 * @return obiekt jest zmodyfikowany (true) lub nie (false).
 */
  public void setModified(boolean val) {
    m_modified = val;
  }

/**
 * Sprawdza czy blok zostal zmodyfikowany.
 * @return blok jest zmodyfikowany (true) lub nie (false).
 */
  public boolean isModified() {
    return m_modified;
  }

/**
 * Zwraca wielkosc bloku.
 * @return wielkosc bloku.
 */
  public abstract short getSize();

/**
 * Serializuje blok.
 * @return zserializowany blok.
 */
  public abstract byte[] toByteArray();
}
