package pl.edu.pjwstk.yaod.store;

import java.io.*;

import pl.edu.pjwstk.yaod.atoms.*;

/**
 * Klasa z ktorej dziedzicza wszystkie sklady.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public abstract class AbstractStore {
  private short m_id;

/**
 * Tworzy nowy sklad.
 * @param id identyfikator nowego skladu.
 * @return nowy sklad.
 */
  public AbstractStore(short id) {
    m_id = id;
  }

/**
 * Zwraca identyfikator skladu.
 * @return comments
 */
  public short getStoreID() {
    return m_id;
  }
  
/**
 * Tworzy obiekt oslony przekazywany do procesu SVRP.
 * @return utworzona oslona.
 */
  public abstract Wrappable createClient();

/**
 * Otwiera sklad.
 * @exception FileNotFoundException nie mozna otworzyc skladu.
 */
  public abstract void open() throws FileNotFoundException;

/**
 * Zamyka sklad.
 * @exception IOException blad przy zamykaniu skladu.
 */
  public abstract void close() throws IOException;

/**
 * Inicjalizuje sklad (np. sformatowanie plikow danych).
 * @exception Exception blad inicjalizacji.
 */
  public abstract void initialize() throws Exception;

/**
 * Ustala identyfikator obiektu wejsciowego do skladu.
 * @param entry ustala identyfikator atomu wejsciowego.
 */
  public abstract void setEntry(AtomID entry);

/**
 * Zwraca identyfikator atomu wejsciowego do skladu.
 * @return identyfikator atomu wejsciowego.
 */
  public abstract AtomID getEntry();
}
