package pl.edu.pjwstk.yaod.store;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.store.defaultstore.blocks.*;

/**
 * Interfejs implementowany przez wszystkie oslony skladow.
 * Oslona tworzona jest przez sklad i wywoluje metody tego skladu.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public interface Wrappable {
/**
 * Zwraca identyfikator atomu wejsciowego do skladu.
 * @return identyfikator atomu wejsciowego.
 */
  public AtomID getEntry();

/**
 * Zwraca atom wkazany za pomoca jego identyfikatora.
 * @param ptr identyfikator atomu.
 * @exception NoSuchAtomException brak takiego atomu w skladzie.
 * @return odczytany atom.
 */
  public Atom retrieveAtom(AtomID ptr) throws NoSuchAtomException;

/**
 * Kasuje atom wskazany podanym identyfikatorem.
 * @param ptr identyfikator kasowanego atomu.
 * @exception NoSuchAtomException brak takiego atomu w skladzie.
 */
  public void deleteAtom(AtomID ptr) throws NoSuchAtomException;

/**
 * Aktualizuje atom zapisany wczesniej w skladzie.
 * @param atm atom do zaktualizowania.
 * @exception BlockFreeSpaceException zbyt malo wolnej przestrzeni.
 */
  public void updateAtom(Atom atm) throws BlockFreeSpaceException;

/**
 * Tworzy nowy atom w skladzie.
 * @param atm atom ktory ma zostac zapisany w skladzie.
 * @exception BlockFreeSpaceException zbyt malo wolnej przestrzeni.
 */
  public void createAtom(Atom atm) throws BlockFreeSpaceException;
}
