package pl.edu.pjwstk.yaod.store.defaultstore.persistentstore;

import java.io.*;

/**
 * Klasa reprezentuje pliki danych.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class Datafile {
  private RandomAccessFile m_file;
  private DatafileDescriptor m_fileInfo;

/**
 * Tworzy nowy obiekt pliku danych.
 * @param info deskryptor pliku danych.
 * @return obiekt pliku danych.
 */
  public Datafile(DatafileDescriptor info) {
    m_fileInfo = info;
  }

/**
 * Zwraca deksryptor pliku danych.
 * @return deskryptor pliku danych.
 */
  public DatafileDescriptor getDatafileDescriptor() {
    return m_fileInfo;
  }

/**
 * Sprawdza czy plik jest otwarty.
 * @return plik jest otwarty (true) lub nie (false).
 */
  public boolean isOpen() {
    return m_file == null ? false : true;
  }

/**
 * Zwraca ilosc blokow jakie zawiera plik danych.
 * @return liczba blokow danych.
 */
  public int holdsBlocks() {
    return m_fileInfo.getFileSize() / m_fileInfo.getBlockSize();
  }

/**
 * Otwiera plik danych.
 * @param mode tryb otwarcia ("r", "rw" itd.).
 * @exception FileNotFoundException brak takiego pliku.
 */
  public void open(String mode) throws FileNotFoundException {
    if (isOpen() == false) {
      System.out.println("*** Opening datafile " + m_fileInfo.getPath());
      m_file = new RandomAccessFile(m_fileInfo.getPath(), mode);
    }
  }

/**
 * Zamyka plik danych.
 * @exception IOException nie mozna zamknac pliku danych.
 */
  public void close() throws IOException {
    if (isOpen() == true) {
      System.out.println("*** Closing datafile " + m_fileInfo.getPath());
      m_file.close();
    }
  }

/**
 * Wczytuje do podanego bufora dane poczawszy od podanego offsetu.
 * @param buf gdzie zapisac dane.
 * @param offset od jakiego punktu czytac dane (dlugosc rowna buf.length).
 * @exception IOException nie mozna wczytac danych.
 */
  public void read(byte[] buf, int offset) throws IOException {
    m_file.seek(offset);
    m_file.read(buf, 0, buf.length);
  }

/**
 * Zapisuje zawartosc podanej tablicy na podanej pozycji w pliku.
 * @param buf dane do zapisania.
 * @param pozycja na ktorej zapisac dane.
 * @exception nie mozna zapisac danych.
 */
  public void write(byte[] buf, int offset) throws IOException {
    m_file.seek(offset);
    m_file.write(buf);
  }
}
