package pl.edu.pjwstk.yaod.store.defaultstore.persistentstore;

import java.io.*;

/**
 * Klasa reprezentuje deskryptor pliku danych. Obiekt tej klasy
 * opisuje najwazniejsze wlasciwosci pliku danych i jest serializowany
 * razem z deskryptorem skladu. Deserializujac taki obiekt sklad
 * wie jakie pliki danych do niego naleza.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class DatafileDescriptor implements Serializable {
  private String m_path;
  private int m_fileSize;
  private short m_blockSize;
  private short m_id;

/**
 * Tworzy nowy deskryptor pliku danych. 
 * @param id identyfikator pliku danych.
 * @param path sciezka dostepu do pliku danych.
 * @param fileSize rozmiar pliku.
 * @param blockSize rozmiar bloku danych.
 * @return nowy deskryptor pliku danych.
 */
  public DatafileDescriptor(short id, String path, int fileSize, short blockSize) {
    m_id = id;
    m_path = path;
    m_fileSize = fileSize;
    m_blockSize = blockSize;
  }

/**
 * Zwraca identyfikator pliku danych.
 * @return identyfikator pliku danych.
 */
  public short getFileID() {
    return m_id;
  }

/**
 * Zwraca rozmiar bloku danych wykorzystywanego przez plik.
 * @return wielkosc bloku.
 */
  public short getBlockSize() {
    return m_blockSize;
  }

/**
 * Zwraca rozmiar pliku danych.
 * @return rozmiar pliku.
 */
  public int getFileSize() {
    return m_fileSize;
  }

/**
 * Zwraca sciezke dostepu do pliku.
 * @return sciezka dostepu.
 */
  public String getPath() {
    return m_path;
  }
}
