package pl.edu.pjwstk.yaod.store.defaultstore.persistentstore;

import pl.edu.pjwstk.yaod.atoms.*;

/**
 * Klasa reprezentuje identyfikator atomu pochodzacego ze skladu
 * trwalych obiektow. Identyfikator taki ma postac <sklad, plik, blok, slot>.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class PersistentAtomID extends PersistentRecordID implements AtomID, java.io.Serializable {
/**
 * Konstruktor tworzacy nowy identyfikator atomu. 
 * @param store identyfikator skladu.
 * @param file identyfikator pliku danych.
 * @param block identyfikator bloku danych.
 * @param slot identyfikator slotu
 * @return nowy identyfikator atomu.
 */
  public PersistentAtomID(short store, short file, short block, short slot) {
    super(new PersistentBlockID(store, file, block), slot);
  }

/**
 * Zwraca identyfikator skladu.
 * @return id skladu.
 */
  public short getStoreID() {
    return getBlockID().getStoreID();
  }

/**
 * Zwraca klucz haszowania dla tego identyfikatora (potrzebne
 * do bufora atomow, ktory jest obiektem Hashtable).
 * @return klucz haszowania.
 */
  public int hashCode() {
    PersistentBlockID blockID = (PersistentBlockID) getBlockID();

    return  blockID.getStoreID() * 1000000 +
            blockID.getFileID() * 10000 +
            blockID.getBlockID() * 100 +
            getSlotID();
  }

/**
 * Sprawdza czy dwa identyfikatory dotycza tego samego atomu.
 * @param par porownywany obiekt.
 * @return identyfikatory posiadaja te same wartosci (true) lub nie (false).
 */
  public boolean equals(Object par) {
    if (par instanceof PersistentAtomID == false)
      return false;

    PersistentAtomID atomID2 = (PersistentAtomID) par;
  
    PersistentBlockID blockID1 = (PersistentBlockID) ((PersistentAtomID) par).getBlockID();
    PersistentBlockID blockID2 = (PersistentBlockID) getBlockID();
    
    return blockID1.getBlockID() == blockID2.getBlockID() && 
           blockID1.getFileID() == blockID2.getFileID() && 
           blockID1.getStoreID() == blockID2.getStoreID() &&
           this.getSlotID() == atomID2.getSlotID();
  }

/**
 * Zapisuje identyfikator w formie umozliwijajacej wyswietlenie jej na ekranie.
 * @return identyfikator atomu jako ciag znakow.
 */
  public String toString() {
    PersistentBlockID blockID = (PersistentBlockID) getBlockID();

    return blockID.getStoreID() + "-" +
           blockID.getFileID() + "-" + 
           blockID.getBlockID() + "-" + 
           getSlotID();
  }
}
