package pl.edu.pjwstk.yaod.store.defaultstore.persistentstore;

import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.util.*;

/**
 * Klasa reprezentuje identyfikator rekordu w trwalym skladzie obiektow.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class PersistentRecordID implements java.io.Serializable {
  private AbstractBlockID m_blockID;
  private short m_slot;

/**
 * Tworzy nowy identyfikator rekordu.
 * @param blockID identyfikator bloku.
 * @param slot numer slotu w bloku.
 * @return nowy identyfikator rekordu.
 */
  public PersistentRecordID(AbstractBlockID blockID, short slot) {
    m_blockID = blockID;
    m_slot = slot;
  }

/**
 * Zwraca numer slotu.
 * @return numer slotu.
 */
  public short getSlotID() {
    return m_slot;
  }

/**
 * Ustala numer slotu.
 * @param id numer slotu.
 */
  public void setSlotID(short id) {
    m_slot = id;
  }

/**
 * Zwraca identyfikator bloku.
 * @return identyfikator bloku.
 */
  public AbstractBlockID getBlockID() {
    return m_blockID;
  }

/**
 * Ustala identyfikator bloku.
 * @param block identyfikator bloku.
 */
  public void setBlockID(AbstractBlockID block) {
    m_blockID = block;
  }


/**
 * Tworzy nowy identyfikator rekordu na bazie jego zserializowanej wersji.
 * @param rawPtr zserializowany identyfikator rekordu.
 * @return zdeserializowany identyfikator rekordu.
 */
  public PersistentRecordID(byte[] rawPtr) {
    short fileID = NewBits.getShort(rawPtr, 0 * NewBits.SHORT_SIZE);
    short blockID = NewBits.getShort(rawPtr, 1 * NewBits.SHORT_SIZE);
    short slotID = NewBits.getShort(rawPtr, 2 * NewBits.SHORT_SIZE);

    m_blockID = (AbstractBlockID) new PersistentBlockID(fileID, blockID);
    m_slot = slotID;
  }

/**
 * Serializuje identyfikator rekordu.
 * @return zserializowany identyfikator rekordu.
 */
  public byte[] toByteArray() {
    byte[] rawPtr = new byte[3 * NewBits.SHORT_SIZE];

    PersistentBlockID blockID = (PersistentBlockID) getBlockID();
    
    NewBits.putShort(rawPtr, 0 * NewBits.SHORT_SIZE, blockID.getFileID());
    NewBits.putShort(rawPtr, 1 * NewBits.SHORT_SIZE, blockID.getBlockID());
    NewBits.putShort(rawPtr, 2 * NewBits.SHORT_SIZE, getSlotID());

    return rawPtr;
  }
}
