package pl.edu.pjwstk.yaod.store.defaultstore.persistentstore;

import java.util.*;

import pl.edu.pjwstk.yaod.store.*;

/**
 * Deskryptor skladu trwalych obiektow. Po zserializowaniu stanowi
 * plik kontrolny skladu. Zawiera informacje konfiguracyjne,
 * np. rozmiar bloku.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class PersistentStoreDescriptor extends AbstractStoreDescriptor {
  private short m_blockSize = 8192;
  private Vector m_datafileDescs;

/**
 * Tworzy nowy deskryptor skladu.
 * @param storeName nazwa skladu.
 * @param storeClass nazwa klasy skladu (potrzebna do dynamicznego ladowania).
 */
  public PersistentStoreDescriptor(String storeName, String storeClass) {
    super(storeName, storeClass);
    m_datafileDescs = new Vector();
  }

/**
 * Ustala rozmiar bloku.
 * @param value rozmiar bloku.
 */
  public void setBlockSize(short value) {
    m_blockSize = value;
  }

/**
 * Zwraca rozmiar bloku.
 * @return rozmiar bloku.
 */
  public short getBlockSize() {
    return m_blockSize;
  }

/**
 * Rejestruje plik danych.
 * @param dfd deskryptor rejestrowanego pliku danych.
 */
  public void registerDatafile(DatafileDescriptor dfd) {
    m_datafileDescs.addElement(dfd);
  }

/**
 * Odrejestrowywuje plik danych.
 * @param desc deskryptor odrejestrowywanego pliku danych.
 */
  public void unregisterDatafile(DatafileDescriptor desc) {
    m_datafileDescs.remove(desc);
  }

/**
 * Zwraca descryptory wszystkich plikow danych.
 * @return zwraca deskryptory zarejestrowanych plikow danych.
 */
  public Enumeration getDatafiles() {
    return m_datafileDescs.elements();
  }
}
