package pl.edu.pjwstk.yaod.store.defaultstore.transientstore;

import java.io.*;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.system.*;
import pl.edu.pjwstk.yaod.util.*;

/**
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */
 
public class TransientStore extends AbstractStore {
  private DatabaseManager m_dbman;
  private String m_ctlFilePath;
  private TransientStoreDescriptor m_descriptor;

  public TransientStore(short id, String ctlPath, DatabaseManager manager) throws Exception {
    super(id);
    
    m_ctlFilePath = ctlPath;
    m_dbman = manager;

    readControlFile();
  }

  public void readControlFile() throws Exception {
    m_descriptor = (TransientStoreDescriptor) new ObjectSerializer().readObject(m_ctlFilePath);
  }

  public void writeControlFile() throws Exception {
    new ObjectSerializer().writeObject(m_descriptor, m_ctlFilePath);
  }

  public void open() throws FileNotFoundException {
  }

  public void initialize() throws Exception {
    System.out.println("*** Initializing transient store ...");

    TransientStoreWrapper wrapper = (TransientStoreWrapper) createClient();

    SpiderAtom entry = new SpiderAtom("entry");
    wrapper.createAtom(entry);

    setEntry(entry.getAID());
  }

  public void close() throws IOException {
  }

  public void setEntry(AtomID entry) {
    m_descriptor.setEntry(entry);
    try {
      writeControlFile();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public AtomID getEntry() {
    return m_descriptor.getEntry();
  }

  public Wrappable createClient() {
    return new TransientStoreWrapper(this);
  }

  public TransientStoreDescriptor getStoreDescriptor() {
    return m_descriptor;
  }

  public short getBlockSize() {
    return 8192;
  }
}