package pl.edu.pjwstk.yaod.system;

import java.io.*;

import java.util.*;

import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.util.*;

/**
 * Klasa reprezentujaca baze danych. Zawiera liste skladow,
 * z ktorych sklada sie baza. W przyszlosci moze zawierac rowniez
 * inne informacje, np. date utworzenia bazy danych. Zserializowana
 * wersja tej klasy i zapisana na dysku tworzy plik kontrolny bazy danych.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class Database implements Serializable {
  private Hashtable m_stores = new Hashtable();

/**
 * Rejestruje sklad w bazie danych.
 * @param ctlPath sciezka dostepu do pliku kontrolnego skladu.
 * @return identyfikator przypisany zarejestrowanemu skladowi.
 */
  public short registerStore(String ctlPath) {
    for (short i = 0; i < Short.MAX_VALUE; i++) {
      if (m_stores.containsKey(new Short(i)) == false) {
        m_stores.put(new Short(i), ctlPath);
        return i;
      }
    }

    return -1;
  }

/**
 * Wyrejestrowywuje sklad z bazy danych.
 * @param storeId identyfikator skladu, ktory nalezy odrejestrowac.
 */

  public void unregisterStore(short storeId) {
    m_stores.remove(new Short(storeId));
  }

/**
 * Wnicjalizuje menedzera danych, wczytujac dla niego pliki kontrolne wszystkich
 * skladow i przesylajac mu instancje tych skladow.
 * @param dbman menedzer danych, ktory ma zostac zainicjalizowany.
 * @exception Exception awaria, np. brak pliku kontrolnego.
 */

  public void initDBManager(DatabaseManager dbman) throws Exception {
    Hashtable stores = new Hashtable();

    for (Enumeration e = m_stores.keys(); e.hasMoreElements(); ) {
      Short key = (Short) e.nextElement();

      String sctlPath = (String) m_stores.get(key);
      AbstractStoreDescriptor desc =
        (AbstractStoreDescriptor) new ObjectSerializer().readObject(sctlPath);

      String className = desc.getStoreClassName();

      Class[] paramClass = { short.class, sctlPath.getClass(), dbman.getClass() };
      Object[] paramValue = { key, sctlPath, dbman };
      AbstractStore store =
        (AbstractStore) Class.forName(className).getConstructor(paramClass).newInstance(paramValue);

      stores.put(key, store);
    }

    dbman.setStores(stores);
  }
}
