package pl.edu.pjwstk.yaod.system;

import java.io.*;

import pl.edu.pjwstk.yaod.util.*;

/**
 * Klasa uruchamiajaca serwer.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class Yaod {
/**
 * Wczytuje z dysku wskazany plik kontrolny bazy danych.
 * @param path sciezka pliku kontrolnego bazy danych.
 * @exception IOException nie mozna odczytac pliku.
 * @exception ClassNotFoundException nieznana klasa zserializowana.
 * @return obiekt Database, bedacy zdeserializowana postacia pliku kontrolnego.
 */
  private Database readControlFile(String path) throws IOException, ClassNotFoundException {
    return (Database) new ObjectSerializer().readObject(path);
  }

/**
 * Uruchamia menedzera bazy danych.
 * @param db baza danych, dla ktorej ma zostac otwarty menedzer.
 * @exception Exception jesli nie mozna uruchomic menedzera.
 */
  private void startup(Database db) throws Exception {
    DatabaseManager dbman = new DatabaseManager(db);
    try {
      dbman.start();
    }
    catch (Exception e) {
      Errors.printError(1, e.getMessage());
      dbman.stop();
    }
  }

/**
 * Startuje baze danych yaod. 
 */
  public final static void main(String args[]) {
    if (args.length == 2 && args[0].equals("-dbctl")) {
      Yaod yaod = new Yaod();

      Database db;

      System.out.println("*** Starting Yaod ...");

      // wczytanie pliku kontrolnego
      try {
        db = yaod.readControlFile(args[1]);
      }
      catch (Exception e) {
        Errors.printError(1, e.getMessage());
        return;
      }

      // uruchomienie menedzera
      try {
        yaod.startup(db);
      }
      catch (Exception e) {
        e.printStackTrace();
        Errors.printError(1, e.getMessage());
      }
    }
    else {
      System.out.println("Parameters:\n -dbctl path - specifies where the database control file is");
    }
  }
}
