package pl.edu.pjwstk.yaod.system.buffers;

import java.util.*;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.system.processes.*;

/**
 * Klasa reprezentuje bufor obiektow. Domyslny sklad trwalych obiektow
 * korzysta operocz tego bufora z bufora stron.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class AtomBuffer {
  private Hashtable m_buffer;
  private DBWR m_dbwr;

/**
 * Tworzy nowy bufor obiektow.
 * @param dbwr proces sekretarza bazy danych, ktory zapisuje
 * obiekty jesli nie ma miejsca w buforze.
 * @return nowy bufor obiektow.
 */
  public AtomBuffer(DBWR dbwr) {
    m_dbwr = dbwr;
    m_buffer = new Hashtable();
  }

/**
 * Zwraca liste wszystkich obiektow zapisanych w buforze.
 * @return lista obiektow.
 */
  public Object[] elements() {
    return m_buffer.values().toArray();
  }

/**
 * Umieszcza obiekt w buforze.
 * @param element obiekt ktory ma zostac umieszczony w buforze.
 */
  public synchronized void put(Atom element) {
    m_buffer.put(element.getAID(), element);
  }

/**
 * Usuwa wszystkie obiekty z bufora.
 */
  public void clear() {
    m_buffer.clear();
  }

/**
 * Zwraca atom o podanym identyfikatorze.
 * @param key identyfikator atomu.
 * @return buforowany atom.
 */
  public Atom get(AtomID key) {
    return (Atom) m_buffer.get(key);
  }

/**
 * Usuwa z buforu atom o podanym identyfikatorze.
 * @param key identyfikator atomu, ktory nalezy usunac z bufora.
 */
  public void remove(AtomID key) {
    m_buffer.remove(key);
  }
}
