package pl.edu.pjwstk.yaod.system.buffers;

import java.util.*;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.system.processes.*;

/**
 * Klasa reprezentuje bufor blokow. Domyslny sklad trwalych obiektow
 * korzysta operocz tego bufora z bufora obiektow.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class BlockBuffer {
  private Hashtable m_buffer;
  private DBWR m_dbwr;

/**
 * Tworzy nowy obiekt bufora blokow.
 * @param dbwr obiekt sekretarza bazy danych majacy zapisywac bloki
 * na nosniku gdy w buforze brakuje miejsca.
 * @return nowy obiekt bufora.
 */
  public BlockBuffer(DBWR dbwr) {
    m_buffer = new Hashtable();
    m_dbwr = dbwr;
  }

/**
 * Usuwa wszystkie obiekty z bufora.
 */
  public void clear() {
    m_buffer.clear();
  }

/**
 * Umieszcza blok w buforze.
 * @param block blok ktory ma zostac umieszczony w buforze.
 */
  public void put(AbstractBlock block) {
    m_buffer.put(block.getBlockID(), block);
  }

/**
 * Usuwa buforowany blok z bufora.
 * @param identyfikator bloku do usuniecia.
 */
  public void remove(AbstractBlockID key) {
    m_buffer.remove(key);
  }

/**
 * Zwraca wszystkie zabuforowane bloki.
 * @return buforowane bloki.
 */
  public Object[] elements() {
    return m_buffer.values().toArray();
  }

/**
 * Zwraca blok o podanym identyfikatorze, jesli blok ten znajduje
 * sie w buforze.
 * @param key identyfikator bloku do pobrania.
 * @return zabuforowany blok.
 */
  public AbstractBlock get(AbstractBlockID key) {
    return (AbstractBlock) m_buffer.get(key);
  }

}
