package pl.edu.pjwstk.yaod.system.processes;

import java.io.*;

import java.net.*;

import pl.edu.pjwstk.yaod.system.*;

/**
 * Proces nasluchu. Jego zadaniem jest wychwytywanie przychodzacych
 * polaczen od klientow i tworzenie dla nich procesow uslugowych.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */
 
public class LSNR extends Thread {
  private DatabaseManager m_dbmanager;
  private boolean m_work;
  private ServerSocket m_serverSocket;
  private int m_protocol;
  private int m_port;
  private String m_protname;

/**
 * Tworzy nowy obiekt procesu sekretarza bazy danych.
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public LSNR(DatabaseManager manager, int protocol, int port) {
    super("lsnr_thread");
    m_dbmanager = manager;
    m_protocol = protocol;
    m_port = port;

    switch (protocol) {
      case SVRP.HTTP : m_protname = "HTTP"; break;
      case SVRP.YNET : m_protname = "YNET"; break;
    }
  }

/**
 * Ustawia flage pracy procesu.
 * @param status pracuj (true), lub zatrzymaj sie (false).
 */
  public void setWork(boolean status) {
    m_work  = status;
  }

/**
 * Startuje watek, otwiera gniazdko serwera.
 * @exception IOException jesli nie mozna utworzyc gniazdka serwera
 * (np. port jest juz wykorzystywany).
 */
  public void startLSNR() throws IOException {
    m_serverSocket = new ServerSocket(m_port);
    m_work = true;

    super.start();    
  }

/**
 * Konczy prace sekretarza bazy danych.
 */
  public void stopLSNR() {
    try {
      m_serverSocket.close();
    }
    catch (IOException e) {}
    m_work = false;
  }

/**
 * Glowna petla watku. Obsluguje zadanie tworzenia procesow uslugowych
 * dla nadchodzacych polaczen od klientow.
 */
  public void run() {
    if (m_serverSocket == null) {
      System.out.println("*** server socket not found");
      return;
    }

    System.out.println("*** LSNR started. Listening on port " + m_port + " for " + m_protname + " requests.");
    while (m_work) {
      Socket clientSocket = null;
      try {
        clientSocket = m_serverSocket.accept();
        m_dbmanager.createServerProcess(clientSocket, m_protocol);
      } catch (IOException e) {
      }
    }
    System.out.println("*** LSNR stopped");
  }
}
