package pl.edu.pjwstk.yaod.util;

import java.io.*;

/**
 * Wykonuje prosta serializacje obiektow. 
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class ObjectSerializer  {
  public ObjectSerializer() {
  }

/**
 * Wczytuje zserializowany obiekt z dysku.
 * @param path sciezka dostepu.
 * @exception IOException blad dysku (np. brak pliku).
 * @exception ClassNotFoundException obiekt bez znanej klasy.
 * @return wczytany obiekt
 */
  public Object readObject(String path) throws java.io.IOException, ClassNotFoundException {
    FileInputStream fi = new FileInputStream(path);
    ObjectInputStream oi = new ObjectInputStream(fi);
    Object obj = oi.readObject();
    oi.close();
    fi.close();

    return obj;
  }

/**
 * Serializuje obiekt i zapisuje go na dysku.
 * @param obj obiekt do zserializowania.
 * @param path gdzie zapisac zserializowany obiekt.
 * @exception IOException blad dysku.
 * @exception ClassNotFoundException obiekt bez klasy
 */
  public void writeObject(Object obj, String path) throws java.io.IOException, ClassNotFoundException {
    FileOutputStream fo = new FileOutputStream(path);
    ObjectOutputStream oo = new ObjectOutputStream(fo);
    oo.writeObject(obj);
    oo.close();
    fo.close();
  }

/**
 * Serializuje obiekt do tablicy byte[]
 * @param obj obiekt do zserializowania.
 * @return tablica byte[] zawierajacac obiekt.
 */
  public byte[] objectSerialize(Object obj) {
    byte[] raw = null;
    try {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      ObjectOutputStream oos = new ObjectOutputStream(baos);
      oos.writeObject(obj);

      raw = baos.toByteArray();

      oos.close();
      baos.close();
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    return raw;
  }

/**
 * Deserializuje obiekt z tablicy byte[].
 * @param raw zserializowany obiekt.
 * @return zdeserialozwany obiekt.
 */
  public Object objectDeserialize(byte[] raw) {
    Object obj = null;
    try {
      ByteArrayInputStream bais = new ByteArrayInputStream(raw);
      ObjectInputStream ois = new ObjectInputStream(bais);
      obj = ois.readObject();
      ois.close();
      bais.close();
    }
    catch (Exception e) {
      System.out.println(e);
    }
    
    return obj;
  }

}