package pl.edu.pjwstk.yaod.workbench.nodes;

import java.awt.event.*;

import java.io.*;

import java.util.*;

import javax.swing.*;
import javax.swing.tree.*;

import pl.edu.pjwstk.yaod.network.*;
import pl.edu.pjwstk.yaod.network.protocol.*;
import pl.edu.pjwstk.yaod.network.protocol.queryresult.*;
import pl.edu.pjwstk.yaod.workbench.*;
import pl.edu.pjwstk.yaod.workbench.details.*;
import pl.edu.pjwstk.yaod.network.YNETCommunicator;

public class DBManagerNode extends AbstractWorkbenchNode {
  private DBManagerPanel m_detailsPanel = new DBManagerPanel();

  private ImageIcon m_disconnectedIcon = new ImageIcon(DBManagerNode.class.getResource("/images/db.gif"));
  private ImageIcon m_connectedIcon = new ImageIcon(DBManagerNode.class.getResource("/images/dbc.gif"));
  private ImageIcon m_icon = m_disconnectedIcon;

  private YNETCommunicator m_comm;

  private DBControlFileNode dbcNode;
  private DBDumpNode dbdNode;
  private XMLDocumentsNode xmlNode;

  String m_name;
  String m_host;
  int m_port;

  boolean m_connected;

  JMenuItem m_connectMenu = new JMenuItem("Connect");
  JMenuItem m_disconnectMenu = new JMenuItem("Disconnect");
  JMenuItem m_unregisterMenu = new JMenuItem("Unregister");
  JSeparator m_separatorMenu = new JSeparator();

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public DBManagerNode(String name, String host, int port) {
    m_name = name;
    m_host = host;
    m_port = port;

    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public String toString() {
    return m_name;
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public Icon getIcon() {
    return m_icon;
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public JPanel getDetailsPanel() {
    return m_detailsPanel;
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public boolean isConnected() {
    return m_connected;
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public void connect() throws IOException {
    m_comm = new YNETCommunicator(m_host, m_port);
    m_connected = true;

    m_icon = m_connectedIcon;
    m_detailsPanel.setConnected(true);

    m_detailsPanel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        executeQuery(m_detailsPanel.getQuery());
      }
    });

    dbdNode = new DBDumpNode();
    this.add(dbdNode);

    xmlNode = new XMLDocumentsNode();
    this.add(xmlNode);
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public void disconnect() {
    if (m_comm != null) {
      m_comm.disconnect();
      m_connected = false;
      m_icon = m_disconnectedIcon;
      m_detailsPanel.setConnected(false);
    }
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public YNETCommunicator getCommunicator() {
    return m_comm;
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public JPopupMenu getPopupMenu() {
    JPopupMenu menu = new JPopupMenu();

    menu.add(isConnected() ? m_disconnectMenu : m_connectMenu);
    menu.add(m_separatorMenu);
    menu.add(m_unregisterMenu);

    return menu;
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public DBManagerNode() {
    try {
      jbInit();
    } catch(Exception e) {
      e.printStackTrace();
    }
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  public void executeQuery(String qry) {
    try {
      m_comm.putMessage(new QueryRequest(qry));
      NetMessage msg = m_comm.getMessage();

      if (msg instanceof ErrorReply) {
        JOptionPane.showMessageDialog(getDetailsPanel(),
          ((ErrorReply) msg).getMessage(), "Query execution error", JOptionPane.ERROR_MESSAGE);

        return;
      }

      QueryReply reply = (QueryReply) msg;

      if (reply == null)
        throw new Exception("Reply is null.");

      DBManagersNode managers = (DBManagersNode) getParent();

      // wyswietlamy sciagniete ast
      if (reply.getAst() != null)
        m_detailsPanel.setAbstractSyntaxTree(reply.getAst());

      // wyswietlamy sciagniety stos srodowiskowy
      if (reply.getAst() != null)
        m_detailsPanel.setEnvironmentalStackTree(reply.getEnvs());

      // wyswietlamy wynik zapytania
      StringBuffer result = new StringBuffer();
      String resStr;

      int i =0;
      for (Enumeration e = reply.getResults(); e.hasMoreElements(); ) {
        if (managers.getShowResultSchema() == managers.XML_RESULTS)
          resStr = ((Result)e.nextElement()).getResultAsXmlString();
        else
          resStr = ((Result)e.nextElement()).getResultAsString();

        result.append(resStr).append("\n");
      }
      m_detailsPanel.setQueryResult(result.toString());

      JOptionPane.showMessageDialog(getDetailsPanel(),
       "Query executed.", "Query execution status", JOptionPane.INFORMATION_MESSAGE);
    }
    catch (Exception e) {
      JOptionPane.showMessageDialog(getDetailsPanel(),
        e.getMessage(), "Query execution error", JOptionPane.ERROR_MESSAGE);
    }
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  private void selfRemove() {
    JTree tree = getTree();
    disconnect();
    removeFromParent();
    ((DefaultTreeModel) tree.getModel()).reload();
  }

/**
 * 
 * @param paramName comments
 * @exception XxxxxxException if ...
 * @return comments
 */
  private void jbInit() throws Exception {
    m_connectMenu.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        for (Enumeration e = getNodeActionListeners(); e.hasMoreElements(); ) {
          NodeActionListener lsnr = (NodeActionListener)e.nextElement();
          lsnr.dbConnectRequested(pl.edu.pjwstk.yaod.workbench.nodes.DBManagerNode.this);
        }
      }
    });

    m_disconnectMenu.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        for (Enumeration e = getNodeActionListeners(); e.hasMoreElements(); ) {
          NodeActionListener lsnr = (NodeActionListener)e.nextElement();
          lsnr.dbDisconnectRequested(pl.edu.pjwstk.yaod.workbench.nodes.DBManagerNode.this);
        }
      }
    });

    m_unregisterMenu.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        selfRemove();
      }
    });
  }
}
