package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree;

import pl.edu.pjwstk.yaod.yaql.visitors.Visitor;
import pl.edu.pjwstk.yaod.yaql.visitors.EvalException;
import java.io.*;

/**
 * The <code>Expression</code> class represents the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public abstract class AbstractSyntaxTree implements Serializable { 

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public abstract String getTypeAsString();

/**
 * Method responsible for traversing the abstract syntax tree structure.
 * @see Visitor
 * @param v the visitor.
 * @exception EvalException if evaluation fails.
 */
  public abstract void accept(Visitor v) throws EvalException;

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public abstract String getSpellingAndPosition();

/**
 * Returns the space string, which size is equal to the name of the longest
 * type name.
 * @return the name of the longest type, replaced by spaces.
 */
  public String getLongestTypeAsSpacesString() {
    String longestTypeName = "TYPE_EXPRESSION_BINARY_NONALGEBRAIC_TRANSITIVECLOSURE";
    String res = "";
    for (int i=0; i<longestTypeName.length(); i++) 
      res += " ";
    return res;
  }

/**
 * Returns the prepared type name for display.
 * @return the prepared type name.
 */
  public String getPreparedTypeAsString() {
    String st = getTypeAsString();
    String longestTypeName = getLongestTypeAsSpacesString();
    int spaces = longestTypeName.length() - st.length();
    for (int i=0; i<spaces; i++)
      st += " ";
    return st;
  }

/**
 * Method responsible for displaying the abstract syntax tree structure.
 * @param st the string which is an auxiliary tabulator.
 */
  public abstract void show(String st);

/**
 * Method responsible for displaying the abstract syntax tree structure.
 */
  public void show() {
    System.out.println("AST objects:");
    show("");
    System.out.println("AST END\n");
  }
}
