package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree;

import java.util.Vector;
import pl.edu.pjwstk.yaod.yaql.visitors.Visitor;
import pl.edu.pjwstk.yaod.yaql.visitors.EvalException;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Statement;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.StatementList;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.IdentifierTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.FormalParametersList;

/**
 * The <code>LastFormalParametersList</code> class represents the tail of formal 
 * parameter list nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class LastFormalParametersList extends FormalParametersList {
  public IdentifierTerminal tail;

/**
 * Creates a new last formal parameters list node with the given terminal 
 * <code>tail</code>.
 * @param tail identifier terminal.
 */
  public LastFormalParametersList(IdentifierTerminal tail) { 
    this.tail = tail; 
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_FORMALPARAMETERS_LAST";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (tail.getSpelling() + " " + tail.getPosition());
  }
  public void accept(Visitor v) throws EvalException {
//    v.visit(this);
  }

  public int getNumberOfParameters(Vector v) {
    v.add(tail);
    return 1;
  }

/**
 * Method responsible for displaying the unary node.
 * @param st the string which is an auxiliary tabulator.
 */
  public void show(String st) { 
    System.out.print(" [ " + getPreparedTypeAsString() + " ]\t");
    System.out.println(st + getSpellingAndPosition());
//    tail.show(st + "  "); 
  }
}