package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree;

import pl.edu.pjwstk.yaod.yaql.visitors.Visitor;
import pl.edu.pjwstk.yaod.yaql.visitors.EvalException;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Statement;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.StatementList;

/**
 * The <code>LastStatementList</code> class represents the tail of  
 * statement list nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class LastStatementList extends StatementList {
  public Statement tail;

/**
 * Creates a new last statement list node with the given statement 
 * <code>tail</code>.
 * @param tail any statement.
 */
  public LastStatementList(Statement tail) { 
    this.tail = tail; 
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_STATEMENTLIST_LAST";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return "\\";
  }

/**
 * Method responsible for traversing the abstract syntax tree structure.
 * @see Visitor
 * @param v the visitor.
 * @exception EvalException if evaluation fails.
 */
  public void accept(Visitor v) throws EvalException {
    v.visit(this);
  }

/**
 * Method responsible for displaying the last statement list node.
 * @param st the string which is an auxiliary tabulator.
 */
  public void show(String st) { 
    System.out.print(" [ " + getPreparedTypeAsString() + " ]\t");
    System.out.println(st + getSpellingAndPosition());
    tail.show(st + "  "); 
  }
}