package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree;

import java.util.Vector;
import pl.edu.pjwstk.yaod.yaql.visitors.Visitor;
import pl.edu.pjwstk.yaod.yaql.visitors.EvalException;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Statement;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.StatementList;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.IdentifierTerminal;

/**
 * The <code>LastFormalParametersList</code> class represents the pair of formal 
 * parameter list nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class PairFormalParametersList extends FormalParametersList {
  public IdentifierTerminal head; 
  public FormalParametersList tail;

/**
 * Creates a new pair formal parameters list node with the given formal 
 * parameters list <code>tail</code> and terminal <code>head</code>.
 * @param tail any formal parameters list.
 * @param head identifier terminal.
 */
  public PairFormalParametersList(IdentifierTerminal head, FormalParametersList tail) { 
    this.head = head; 
    this.tail = tail; 
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_FORMALPARAMETERS_PAIR";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (head.getSpelling() + " " + head.getPosition());
  }
  public int getNumberOfParameters(Vector v) {
    v.add(head);
    return 1 + tail.getNumberOfParameters(v);
  }
  public void accept(Visitor v) throws EvalException {
//    v.visit(this);
  }

/**
 * Method responsible for displaying the unary node.
 * @param st the string which is an auxiliary tabulator.
 */
  public void show(String st) { 
    System.out.print(" [ " + getPreparedTypeAsString() + " ]\t");
    System.out.println(st + getSpellingAndPosition());
//    head.show(st + "  "); 
    tail.show(st + "  "); 
  }
}