package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree;

import pl.edu.pjwstk.yaod.yaql.visitors.Visitor;
import pl.edu.pjwstk.yaod.yaql.visitors.EvalException;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Statement;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.StatementList;

/**
 * The <code>LastStatementList</code> class represents the pair of  
 * statement list nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class PairStatementList extends StatementList {
  public Statement head; 
  public StatementList tail;

/**
 * Creates a new pair statement list node with the given statement 
 * <code>head</code> and statement list <code>tail</code>.
 * @param head any statement.
 * @param tail any statement list.
 */
  public PairStatementList(Statement head, StatementList tail) { 
    this.head = head; 
    this.tail = tail; 
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_STATEMENTLIST_PAIR";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return "\\";
  }

/**
 * Method responsible for traversing the abstract syntax tree structure.
 * @see Visitor
 * @param v the visitor.
 * @exception EvalException if evaluation fails.
 */
  public void accept(Visitor v) throws EvalException {
    v.visit(this);
  }

/**
 * Method responsible for displaying the pair statement list node.
 * @param st the string which is an auxiliary tabulator.
 */
  public void show(String st) { 
    System.out.print(" [ " + getPreparedTypeAsString() + " ]\t");
    System.out.println(st + getSpellingAndPosition());
    head.show(st + "  "); 
    tail.show(st + "  "); 
  }
}