package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions;

import pl.edu.pjwstk.yaod.yaql.visitors.Visitor;
import pl.edu.pjwstk.yaod.yaql.visitors.EvalException;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Expression;

/**
 * The <code>TernaryExpression</code> class represents the ternaryexpression 
 * nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public abstract class TernaryExpression extends Expression {
  public Expression l, m, r;

/**
 * Creates a new ternary expression node with the given expressions <code>l</code>, 
 * <code>m</code> and <code>r</code>.
 * @param l any expression.
 * @param m any expression.
 * @param r any expression.
 */
  public TernaryExpression (Expression l, Expression m, Expression r) {
    this.l  = l;
    this.m = m;
    this.r = r;
  }

/**
 * Method responsible for traversing the abstract syntax tree structure.
 * @see Visitor
 * @param v the visitor.
 * @exception EvalException if evaluation fails.
 */
  public void accept(Visitor v) throws EvalException {
    v.visit(this);
  }

/**
 * Method responsible for displaying the ternary node.
 * @param st the string which is an auxiliary tabulator.
 */
  public void show(String st) {
    System.out.print(" [ " + getPreparedTypeAsString() + " ]\t");
    System.out.println(st +  getSpellingAndPosition());
    l.show(st + "  ");
    m.show(st + "  ");
    r.show(st + "  ");
  }
}
