package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.binary;

import pl.edu.pjwstk.yaod.yaql.visitors.Visitor;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.OperatorTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.AlgebraicExpression;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Expression;

/**
 * The <code>AssignExpression</code> class represents the binary 
 * expression nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class AssignExpression extends AlgebraicExpression {
  public OperatorTerminal o;

/**
 * Creates a new binary expression node with the given expressions 
 * <code>l</code> and <code>r</code> and terminal <code>o</code>.
 * @param l any expression.
 * @param r any expression.
 * @param o operator terminal.
 */
  public AssignExpression(Expression l, Expression r, OperatorTerminal o) {
    super(l, r);
    this.o = o;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_EXPRESSION_BINARY_ALGEBRAIC_ASSIGN";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (o.getSpelling() + " " + o.getPosition());
  }
}