package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.binary;

import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.OperatorTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.AlgebraicExpression;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Expression;

/**
 * The <code>BRangeExpression</code> class represents the binary 
 * expression nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class BRangeExpression extends AlgebraicExpression {
  OperatorTerminal lo, ro;

/**
 * Creates a new binary expression node with the given expressions 
 * <code>l</code> and <code>r</code> and terminals <code>lo</code> and
 * <code>ro</code>.
 * @param l any expression.
 * @param r any expression.
 * @param lo operator terminal.
 * @param ro operator terminal.
 */
  public BRangeExpression(Expression l, Expression r, 
                         OperatorTerminal lo, OperatorTerminal ro) {
    super(l, r);
    this.lo = lo;
    this.ro = ro;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_EXPRESSION_BINARY_ALGEBRAIC_RANGE";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (lo.getSpelling() + " " + lo.getPosition() + " " +
            ro.getSpelling() + " " + ro.getPosition());
  }
}
