package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.binary;

import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.KeywordTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.NonAlgebraicExpression;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Expression;

/**
 * The <code>ForAnyExpression</code> class represents the binary 
 * expression nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class ForAnyExpression extends NonAlgebraicExpression {
  KeywordTerminal lk, rk;

/**
 * Creates a new binary expression node with the given expressions 
 * <code>l</code> and <code>r</code> and terminal <code>lk</code> and
 * <code>rk</code>.
 * @param l any expression.
 * @param r any expression.
 * @param lk keyword terminal.
 * @param rk keyword terminal.
 */
  public ForAnyExpression(Expression l, Expression r, KeywordTerminal lk, KeywordTerminal rk) {
    super(l, r);
    this.lk = lk;
    this.rk = rk;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_EXPRESSION_BINARY_NONALGEBRAIC_FORANY";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (lk.getSpelling() + " " + lk.getPosition() + " " + 
            rk.getSpelling() + " " + rk.getPosition());
  }
}
