package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.binary;

import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.KeywordTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Expression;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.NonAlgebraicExpression;

/**
 * The <code>SelectionExpression</code> class represents the binary 
 * expression nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class SelectionExpression extends NonAlgebraicExpression {
  KeywordTerminal k;

/**
 * Creates a new binary expression node with the given expressions 
 * <code>l</code> and <code>r</code> and terminal <code>k</code>.
 * @param l any expression.
 * @param r any expression.
 * @param k keyword terminal.
 */
  public SelectionExpression(Expression l, Expression r, KeywordTerminal k) {
    super(l, r);
    this.k = k;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_EXPRESSION_BINARY_NONALGEBRAIC_SELECTION";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (k.getSpelling() + " " + k.getPosition());
  }
}