package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.ternary;

import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.OperatorTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Expression;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.TernaryExpression;

/**
 * The <code>RangeExpression</code> class represents the ternary 
 * range expression nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class RangeExpression extends TernaryExpression {
  OperatorTerminal lo, mo, ro;

/**
 * Creates a new ternary expression node with the given expressions 
 * <code>l</code>, <code>m</code> and <code>r</code> and terminals 
 * <code>lo</code>, <code>mo</code> and <code>ro</code>.
 * @param l any expression.
 * @param m any expression.
 * @param r any expression.
 * @param lo operator terminal.
 * @param mo operator terminal.
 * @param ro operator terminal.
 */
  public RangeExpression(Expression l, Expression m, Expression r, 
                    OperatorTerminal lo, OperatorTerminal mo, OperatorTerminal ro) {
    super(l, m, r);
    this.lo = lo;
    this.mo = mo;
    this.ro = ro;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_EXPRESSION_TERNARY_RANGE";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (lo.getSpelling() + " " + lo.getPosition() + " " + 
            mo.getSpelling() + " " + mo.getPosition() + " " +
            ro.getSpelling() + " " + ro.getPosition());
  }
}


