package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.unary;

import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.IdentifierTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.KeywordTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.ParametricExpression;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Expression;

/**
 * The <code>AsExpression</code> class represents the unary 
 * expression nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class AsExpression extends ParametricExpression {
  KeywordTerminal k;

/**
 * Creates a new unary expression node with the given expression <code>r</code>
 * and terminall <code>k</code> and <code>i</code>.
 * @param r any expression.
 * @param k keyword terminal.
 * @param i identifier terminal.
 */
  public AsExpression(Expression l, KeywordTerminal k, IdentifierTerminal i) {
    super(l);
    this.k = k;
    this.i = i;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_EXPRESSION_UNARY_PARAMETRIC_AS";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (k.getSpelling() + " " + k.getPosition() + " " +
            i.getSpelling() + " " + i.getPosition());
  }
}


