package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.unary;

import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.KeywordTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Expression;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.NonParametricExpression;

/**
 * The <code>CreateLocalExpression</code> class represents the unary 
 * expression nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class CreateLocalExpression extends NonParametricExpression {
  KeywordTerminal lk, rk;

/**
 * Creates a new unary expression node with the given expression <code>r</code>
 * and terminall <code>lk</code> and <code>rk</code>.
 * @param r any expression.
 * @param lk keyword terminal.
 * @param rk keyword terminal.
 */
  public CreateLocalExpression(Expression r, KeywordTerminal lk, KeywordTerminal rk) {
    super(r);
    this.lk = lk;
    this.rk = rk;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_EXPRESSION_UNARY_NONPARAMETRIC_CREATELOCAL";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (lk.getSpelling() + " " + lk.getPosition() + " " +
            rk.getSpelling() + " " + rk.getPosition());
  }
}