package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.unary;

import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.IdentifierTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.KeywordTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.ParametricExpression;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Expression;

/**
 * The <code>GroupAsExpression</code> class represents the unary 
 * expression nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class GroupAsExpression extends ParametricExpression {
  KeywordTerminal lk, rk;

/**
 * Creates a new unary expression node with the given expression <code>r</code>
 * and terminals <code>lk</code>, <code>rk</code> and <code>i</code>.
 * @param r any expression.
 * @param lk keyword terminal.
 * @param rk keyword terminal.
 * @param i identifier terminal.
 */
  public GroupAsExpression(Expression l, KeywordTerminal lk, KeywordTerminal rk, IdentifierTerminal i) {
    super(l);
    this.lk = lk;
    this.rk = rk;
    this.i = i;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_EXPRESSION_UNARY_PARAMETRIC_GROUPAS";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (lk.getSpelling() + " " + lk.getPosition() + " " +
            rk.getSpelling() + " " + rk.getPosition() + " " +
            i.getSpelling()  + " " + i.getPosition());
  }
}
