package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.unary;

import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.IdentifierTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Expression;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.NonParametricExpression;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.binary.*;
import java.util.*;

/**
 * The <code>ProcedureCallExpression</code> class represents the unary 
 * expression nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class ProcedureCallExpression extends NonParametricExpression {
  public IdentifierTerminal i;

/**
 * Creates a new unary expression node with the given expression <code>r</code>
 * and terminal <code>i</code>.
 * @param r any expression.
 * @param i identifier terminal.
 */
  public ProcedureCallExpression(Expression r, IdentifierTerminal i) {
    super(r);
    this.i = i;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_EXPRESSION_UNARY_NONPARAMETRIC_PROCEDURECALL";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return i.getSpelling() + "(...) " + i.getPosition();
  }

  public int getNumberOfParameters(Vector v) {
    return getNumberOfParameters(r, v);
  }
  public int getNumberOfParameters(Expression tmp, Vector v) {
    if (tmp instanceof ListExpression) {
      int a = getNumberOfParameters(((ListExpression)tmp).l, v) + 1;
      v.add(((ListExpression)tmp).r);
      return a;
    } else {
      v.add(tmp);
      return 1;
    }
  }
}