package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.statements;

import pl.edu.pjwstk.yaod.yaql.visitors.Visitor;
import pl.edu.pjwstk.yaod.yaql.visitors.EvalException;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Statement;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.KeywordTerminal;

/**
 * The <code>EmptyBlockStatement</code> class represents the 
 * statement nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class EmptyBlockStatement extends Statement {
  KeywordTerminal lk, rk;

/**
 * Creates a new statement node with the given terminals <code>lk</code> and
 * <code>rk</code>.
 * @param lk keyword terminal.
 * @param rk keyword terminal.
 */
  public EmptyBlockStatement(KeywordTerminal lk, KeywordTerminal rk) {
    this.lk = lk;
    this.rk = rk;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_STATEMENT_EMPTYBLOCK";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (lk.getSpelling() + " " + lk.getPosition() + " " +
            rk.getSpelling() + " " + rk.getPosition());
  }

/**
 * Method responsible for traversing the abstract syntax tree structure.
 * @see Visitor
 * @param v the visitor.
 * @exception EvalException if evaluation fails.
 */
  public void accept(Visitor v) throws EvalException {
    v.visit(this);
  }

/**
 * Method responsible for displaying the statement node.
 * @param st the string which is an auxiliary tabulator.
 */
  public void show(String st) {
    System.out.print(" [ " + getPreparedTypeAsString() + " ]\t");
    System.out.println(st + getSpellingAndPosition());
  }
}