package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.statements;

import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.*;
import pl.edu.pjwstk.yaod.yaql.visitors.*;

/**
 * The <code>EmptyProcedureDeclarationStatement</code> class represents the 
 * statement nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class EmptyProcedureDeclarationStatement extends Statement {
  public StatementList s;
  public KeywordTerminal k;
  public IdentifierTerminal i;

/**
 * Creates a new statement node with the given statement list <code>s</code>
 * and terminals <code>k</code> and <code>i</code>.
 * @param s any statement.
 * @param k keyword terminal.
 * @param i identifier terminal.
 */
  public EmptyProcedureDeclarationStatement(StatementList s, KeywordTerminal k, IdentifierTerminal i) {
    this.s = s;
    this.k = k;
    this.i = i;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_STATEMENT_EMPTYPROCEDUREDECLARATION";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (k.getSpelling() + " " + k.getPosition() + " " +
            i.getSpelling() + "() " + i.getPosition());
  }

/**
 * Method responsible for traversing the abstract syntax tree structure.
 * @see Visitor
 * @param v the visitor.
 * @exception EvalException if evaluation fails.
 */
  public void accept(Visitor v) throws EvalException {
    v.visit(this);
  }

/**
 * Method responsible for displaying the statement node.
 * @param st the string which is an auxiliary tabulator.
 */
  public void show(String st) {
    System.out.print(" [ " + getPreparedTypeAsString() + " ]\t");
    System.out.println(st + getSpellingAndPosition());
    s.show(st + "  ");
  }
}