package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.statements;

import pl.edu.pjwstk.yaod.yaql.visitors.Visitor;
import pl.edu.pjwstk.yaod.yaql.visitors.EvalException;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Expression;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Statement;

/**
 * The <code>ExpressionStatement</code> class represents the 
 * statement nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class ExpressionStatement extends Statement {
  public Expression e;

/**
 * Creates a new statement node with the given expression <code>e</code>.
 * @param e any expression.
 */
  public ExpressionStatement(Expression e) {
    this.e = e;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_STATEMENT_EXPRESSION";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return "\\";
  }

/**
 * Method responsible for traversing the abstract syntax tree structure.
 * @see Visitor
 * @param v the visitor.
 * @exception EvalException if evaluation fails.
 */
  public void accept(Visitor v) throws EvalException {
    v.visit(this);
  }

/**
 * Method responsible for displaying the statement node.
 * @param st the string which is an auxiliary tabulator.
 */
  public void show(String st) {
    System.out.print(" [ " + getPreparedTypeAsString() + " ]\t");
    System.out.println(st + getSpellingAndPosition());
    e.show(st + "  ");
  }
}