package pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.statements;

import pl.edu.pjwstk.yaod.yaql.visitors.Visitor;
import pl.edu.pjwstk.yaod.yaql.visitors.EvalException;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Expression;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.KeywordTerminal;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.Statement;

/**
 * The <code>IfElseStatement</code> class represents the 
 * statement nodes in the abstract syntax tree.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class IfElseStatement extends Statement {
  public Expression e;
  public Statement s1, s2;
  public KeywordTerminal lk, rk;

/**
 * Creates a new statement node with the given expression <code>e</code> 
 * statements <code>s1</code> and <code>s2</code> and terminals <code>lk</code>
 * <code>rk</code>.
 * @param e any expression.
 * @param s1 any statement.
 * @param s2 any statement.
 * @param lk keyword terminal.
 * @param rk keyword terminal.
 */
  public IfElseStatement(Expression e, Statement s1, Statement s2, 
                         KeywordTerminal lk, KeywordTerminal rk) {
    this.e = e;
    this.s1 = s1;
    this.s2 = s2;
    this.lk = lk;
    this.rk = rk;
  }

/**
 * Returns the abstract syntax tree node type as a string value.
 * @return the abstract syntax tree node type as string.
 */
  public String getTypeAsString() {
    return "TYPE_STATEMENT_IFELSE";
  }

/**
 * Returns the spelling and positions of all terminals in a node.
 * @return the spelling and positions of all terminals in a node.
 */
  public String getSpellingAndPosition() {
    return (lk.getSpelling() + " " + lk.getPosition() + " " +
            rk.getSpelling() + " " + rk.getPosition());
  }

/**
 * Method responsible for traversing the abstract syntax tree structure.
 * @see Visitor
 * @param v the visitor.
 * @exception EvalException if evaluation fails.
 */
  public void accept(Visitor v) throws EvalException {
    v.visit(this);
  }

/**
 * Method responsible for displaying the statement node.
 * @param st the string which is an auxiliary tabulator.
 */
  public void show(String st) {
    System.out.print(" [ " + getPreparedTypeAsString() + " ]\t");
    System.out.println(st  +  getSpellingAndPosition());
    e.show(st + "  ");
    s1.show(st + "  ");
    s2.show(st + "  ");
  }
}