package pl.edu.pjwstk.yaod.yaql.engine;

import java.util.Vector;
import java.io.PrintWriter;
import pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes.QueryResult;

/**
 * The <code>QueryResultStack</code> class represents the query result stack,
 * which is responsible for holding all possible results which can be yielded 
 * during evaluation.
 * @see QueryResult
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class QueryResultStack {
  private Vector results;

/**
 * Creates a new query result stack.
 */
  public QueryResultStack() { 
    results = new Vector();
  }

/**
 * Tests whether or not the stack is empty.
 * @return <code>true</code> if stack is empty, <code>false</code> otherwise.
 */
  public boolean empty() {
    return results.isEmpty();
  }

/**
 * Puts new element onto query result stack.
 * @see QueryResult
 */
  public void push(QueryResult qr) {
  	results.add(qr);
  }

/**
 * Returns (and removes) the last element from the query result stack.
 * @see QueryResult
 * @exception EmptyStackException if stack is empty.
 * @return the last section of query result stack.
 */
  public QueryResult pop() throws EmptyStackException {
    if (empty())
 	    throw new EmptyStackException("QueryResultStack.pop()");
    return (QueryResult)results.remove(getSize() - 1);
  }

/**
 * Returns (without removal) the last element from the query result stack.
 * @see QueryResult
 * @exception EmptyStackException if stack is empty.
 * @return the last section of query result stack.
 */
  public QueryResult top() throws EmptyStackException {
    if (empty())
 	    throw new EmptyStackException("QueryResultStack.top()");
    return (QueryResult)results.lastElement();
  }

/**
 * Returns the number of elements in the query result stack.
 * @return the number of elements in the query result stack.
 */
  public int getSize() {
  	return results.size(); 
  }

/**
 * Returns (without removal) the i-th element from the query result stack.
 * @see QueryResult
 * @param i the index of element to be returned.
 * @return the i-th element of query result stack.
 */
  public QueryResult getElementAt(int i) { 
  	return (QueryResult)results.get(i);
  }

/**
 * Returns (without removal) the first element from the query result stack.
 * @see QueryResult
 * @return the first section of query result stack.
 */
  public QueryResult getFirstElement() { 
  	return getElementAt(0); 
  }

/**
 * Returns (without removal) the last element from the query result stack.
 * @see QueryResult
 * @return the last section of query result stack.
 */
  public QueryResult getLastElement() { 
  	return getElementAt(getSize()-1); 
  }

/**
 * Shows all the elements of query result stack.
 */
  public void show() {
    System.out.println("QRES objects (QueryResult's = " + getSize() + "):");
    for (int i=0; i<getSize(); i++) {
      QueryResult tmp = getElementAt(i);
      System.out.println("  section " + i + ": ");
      tmp.show("   ", new PrintWriter(System.out));
    }
    System.out.println("QRES END\n");
  }
}
