package pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes;

import pl.edu.pjwstk.yaod.yaql.engine.TypeMismatchException;
import java.io.PrintWriter;
import pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes.SingleValue;
import pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes.QueryResult;

/**
 * The <code>AtomicValue</code> class represents the main class of available
 * atomic results.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public abstract class AtomicValue extends SingleValue {

/**
 * Returns the atomic value as a string value.
 * @return the result type as string.
 */
  public abstract String getValueAsString();

/**
 * Prints the actual atomic result in the raw fasion. The result is printed
 * on <code>out</code> object.
 * @exception TypeMismatchException never in this case.
 * @param st the string which is a auxiliary tabulator.
 * @param out the object responsible for displaying information.
 */
  public void show(String st, PrintWriter out) {
    out.print(" [ " + getPreparedTypeAsString() + " ]\t");
    out.println(st + getValueAsString());
  }


//******************************************************************************
//******************************* UNARY OPERATORS ******************************
//******************************************************************************
/**
 * Prints the actual atomic result in the normal fashion. The result is printed
 * on <code>out</code> object.
 * @exception TypeMismatchException never in this case.
 * @param st the string which is a auxiliary tabulator.
 * @param out the object responsible for displaying information.
 */
	public void operatorPRINT(String st, PrintWriter out) throws TypeMismatchException {
    String tmp = (this instanceof StringValue) ? "\"" : "";
    out.println(st + tmp + getValueAsString() + tmp);
	}

/**
 * Prints the actual atomic result in the XML fasion. The result is printed
 * on <code>out</code> object.
 * @exception TypeMismatchException never in this case.
 * @param st the string which is a auxiliary tabulator.
 * @param out the object responsible for displaying information.
 */
	public void operatorXMLPRINT(String st, PrintWriter out) throws TypeMismatchException {
//    out.println(st + "<ATOMIC>");
    out.println(st + getValueAsString());
//    out.println(st + "</ATOMIC>");
	}

/**
 * Returns the result of dereference operation on atomic result, which is
 * this atomic result itself.
 * @return this atomic value.
 * @exception TypeMismatchException never in this case.
 * @see QueryResult
 */
  public QueryResult operatorDEREFERENCE() throws TypeMismatchException {
    return this;
  }

/**
 * Return the result of full dereference operation on atomic result, which is
 * this atomic result itself.
 * @return this atomic value.
 * @exception TypeMismatchException never in this case.
 * @see QueryResult
 */
  public QueryResult operatorFULLDEREFERENCE() throws TypeMismatchException {
    return this.operatorDEREFERENCE();
  }

/**
 * Return the result of special dereference operation on atomic result, which is
 * this atomic result itself.
 * @return this atomic value.
 * @exception TypeMismatchException never in this case.
 * @see QueryResult
 */
  public QueryResult operatorSPECIALDEREFERENCE() throws TypeMismatchException {
    return this.operatorDEREFERENCE();
  }

/**
 * Return the result of mixed dereference operation on atomic result, which is
 * this atomic result itself.
 * @return this atomic value.
 * @exception TypeMismatchException never in this case.
 * @see QueryResult
 */
  public QueryResult operatorMIXEDDEREFERENCE() throws TypeMismatchException {
    return this.operatorDEREFERENCE();
  }
}