package pl.edu.pjwstk.yaod.yaql.parser;

import pl.edu.pjwstk.yaod.yaql.SBQLException;

/**
 * The <code>ParserException</code> class represents an exception which is
 * thrown if an error occurs during parsing process. It possess
 * the reason of the error in <code>message</code> value.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 * @see SBQLException
 */
public class ParserException extends SBQLException {
  private int line, column;
  
/**
 * Creates a new exception with specified <code>message</code>, additionally
 * with line and column of an error.
 * @param message the message representing the error.
 * @param line the line of the source code in which an error occured.
 * @param column the column of the source code in which an error occured.
 */
  public ParserException(String message, int line, int column) {
    super(message);
    this.line = line;
    this.column = column;
  }

/**
 * Returns the error message.
 * @return the message representing the reason of the error.
 */
  public String getErrorText() {
    return message;
  }

/**
 * Returns the line of an error in the source code.
 * @return the line of an error in the source code.
 */
  public int getErrorLine() {
    return line;
  }

/**
 * Returns the column of an error in the source code.
 * @return the column of an error in the source code.
 */
  public int getErrorColumn() {
    return column;
  }

/**
 * Returns the exception message.
 * @return the message representing the reason of the exception.
 */
  public String getMessage() {
    return message + " at line " + line + ", column " + column;
  }

/**
 * Returns the exception name.
 * @return the message representing the exception name.
 */
  public String getExceptionName() {
    return "PARSER ERROR";
  }
}
