package pl.edu.pjwstk.yaod.yaql.visitors;

import pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes.*;

/**
 * The <code>ReturnException</code> class represents an exception which is
 * thrown during program evaluation, when return statement occurs. It possess
 * the result to be returned in <code>result</code> value.
 * @see ReturnStatement
 * @see QueryResult
 * @see EvalException
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class ReturnException extends EvalException {
  QueryResult result;

/**
 * Creates a new return exception.
 */
  public ReturnException() {
    super(""); 
    result = new BagValue();
  }

/**
 * Creates a new return exception with specified <code>result</code>.
 * @param return the value to be returned by return statement.
 */
  public ReturnException(QueryResult result) {
    super("");
    this.result = result;
  }

/**
 * Returns the exception name.
 * @return the message representing the exception name.
 */
  public String getExceptionName() {
    return "RETURN";
  }

/**
 * Returns the full exception message.
 * @return the message representing the reason of the exception.
 */
  public String getFullMessage() {
    return getExceptionName() + ": " + getMessage();
  }

/**
 * Return the value to be returned by return statement.
 * @return the value to be returned by return statement.
 */
  public QueryResult getReturnValue() {
    return result;
  }
}